# encoding: utf-8

"""Gherkin step implementations for shape-related features."""

from __future__ import unicode_literals

import hashlib

from behave import given, when, then

from pptx import Presentation
from pptx.enum.shapes import MSO_SHAPE, MSO_SHAPE_TYPE, PP_MEDIA_TYPE
from pptx.action import ActionSetting
from pptx.util import Emu

from helpers import cls_qname, test_file, test_pptx


# given ===================================================


@given("an autoshape")
def given_an_autoshape(context):
    prs = Presentation(test_pptx("shp-autoshape-adjustments"))
    context.shape = prs.slides[0].shapes[0]


@given("(builder._start_x, builder._start_y) is ({x_str}, {y_str})")
def given_builder_start_x_builder_start_y_is_x_y(context, x_str, y_str):
    builder = context.builder
    builder._start_x, builder._start_y = int(x_str), int(y_str)


@given("(builder._x_scale, builder._y_scale) is ({p_str}, {q_str})")
def given_builder_x_scale_builder_y_scale_is_p_q(context, p_str, q_str):
    builder = context.builder
    builder._x_scale, builder._y_scale = float(p_str), float(q_str)


@given("a chevron shape")
def given_a_chevron_shape(context):
    prs = Presentation(test_pptx("shp-autoshape-adjustments"))
    context.shape = prs.slides[0].shapes[0]


@given("a Connector object as shape")
def given_a_Connector_object_as_shape(context):
    prs = Presentation(test_pptx("shp-common-props"))
    sld = prs.slides[0]
    context.shape = sld.shapes[4]


@given("a connector and a 1 inch square picture at 0, 0")
def given_a_connector_and_a_1_inch_square_picture_at_0_0(context):
    prs = Presentation(test_pptx("shp-connector-props"))
    shapes = prs.slides[1].shapes
    context.picture = shapes[0]
    context.connector = shapes[1]


@given("a connector having its begin point at ({x}, {y})")
def given_a_connector_having_its_begin_point_at_x_y(context, x, y):
    prs = Presentation(test_pptx("shp-connector-props"))
    sld = prs.slides[0]
    context.connector = sld.shapes[0]


@given("a connector having its end point at ({x}, {y})")
def given_a_connector_having_its_end_point_at_x_y(context, x, y):
    prs = Presentation(test_pptx("shp-connector-props"))
    sld = prs.slides[0]
    context.connector = sld.shapes[0]


@given("an empty GroupShape object as shape")
def given_an_empty_GroupShape_object_as_shape(context):
    prs = Presentation(test_pptx("shp-common-props"))
    sld = prs.slides[0]
    context.shape = sld.shapes.add_group_shape()


@given("a FreeformBuilder object as builder")
def given_a_FreeformBuilder_object_as_builder(context):
    shapes = Presentation(test_pptx("shp-freeform")).slides[0].shapes
    builder = shapes.build_freeform()
    context.builder = builder


@given("a GraphicFrame object as shape")
def given_a_GraphicFrame_object_as_shape(context):
    # shouldn't matter, but this one contains a table
    prs = Presentation(test_pptx("shp-common-props"))
    sld = prs.slides[0]
    context.shape = sld.shapes[2]


@given("a GraphicFrame object containing a chart as shape")
def given_a_GraphicFrame_object_containing_a_chart_as_shape(context):
    prs = Presentation(test_pptx("shp-access-chart"))
    sld = prs.slides[0]
    context.shape = sld.shapes[0]


@given("a GraphicFrame object containing a table as shape")
def given_a_GraphicFrame_object_containing_a_table_as_shape(context):
    prs = Presentation(test_pptx("shp-access-chart"))
    sld = prs.slides[1]
    context.shape = sld.shapes[0]


@given("a GraphicFrame object containing an OLE object as shape")
@given("a GraphicFrame object containing an embedded XLSX object as shape")
def given_a_GraphicFrame_object_containing_an_embedded_xlsx_object_as_shape(context):
    prs = Presentation(test_pptx("shp-access-ole-object"))
    sld = prs.slides[0]
    context.shape = sld.shapes[0]


@given("a GroupShape object as group_shape")
def given_a_GroupShape_object_as_group_shape(context):
    prs = Presentation(test_pptx("shp-groupshape"))
    sld = prs.slides[0]
    context.group_shape = sld.shapes[0]


@given("a GroupShape object as shape")
def given_a_GroupShape_object_as_shape(context):
    prs = Presentation(test_pptx("shp-common-props"))
    sld = prs.slides[0]
    context.shape = sld.shapes[3]


@given("a movie shape")
def given_a_movie_shape(context):
    prs = Presentation(test_pptx("shp-movie-props"))
    context.movie = prs.slides[0].shapes[0]


@given("an _OleFormat object for an OLE object as ole_format")
@given("an _OleFormat object for an embedded XLSX as ole_format")
def given_an_OleFormat_object_for_an_embedded_XLSX_as_ole_format(context):
    prs = Presentation(test_pptx("shp-access-ole-object"))
    context.ole_format = prs.slides[0].shapes[0].ole_format


@given("a Picture object as picture")
def given_a_Picture_object_as_picture(context):
    slide = Presentation(test_pptx("shp-picture")).slides[0]
    context.picture = slide.shapes[0]


@given("a Picture object as shape")
def given_a_Picture_object_as_shape(context):
    slide = Presentation(test_pptx("shp-common-props")).slides[0]
    context.shape = slide.shapes[1]


@given("a Picture object with {crop_or_no} as picture")
def given_a_Picture_object_with_crop_or_no_as_picture(context, crop_or_no):
    shape_idx = {"no cropping": 0, "cropping": 1}[crop_or_no]
    slide = Presentation(test_pptx("shp-picture")).slides[0]
    context.picture = slide.shapes[shape_idx]


@given("a rotated {shape_type} object as shape")
def given_a_rotated_shape_type_object_as_shape(context, shape_type):
    shape_idx = {
        "Shape": 0,
        "Picture": 1,
        "GraphicFrame": 2,
        "GroupShape": 3,
        "Connector": 4,
    }[shape_type]
    prs = Presentation(test_pptx("shp-common-props"))
    sld = prs.slides[1]
    context.shape = sld.shapes[shape_idx]


@given("a Shape object as shape")
def given_a_Shape_object_as_shape(context):
    prs = Presentation(test_pptx("shp-common-props"))
    sld = prs.slides[0]
    context.shape = sld.shapes[0]


@given("a Shape object having text as shape")
def given_a_Shape_object_having_text_as_shape(context):
    prs = Presentation(test_pptx("shp-autoshape-props"))
    context.shape = prs.slides[0].shapes[0]


@given("a {shape_type} object on a slide as shape")
def given_a_shape_on_a_slide(context, shape_type):
    shape_idx = {
        "Shape": 0,
        "Picture": 1,
        "GraphicFrame": 2,
        "GroupShape": 3,
        "Connector": 4,
    }[shape_type]
    prs = Presentation(test_pptx("shp-common-props"))
    sld = prs.slides[0]
    context.shape = sld.shapes[shape_idx]
    context.slide = sld


@given("a textbox")
def given_a_textbox(context):
    prs = Presentation(test_pptx("shp-common-props"))
    sld = prs.slides[0]
    context.shape = sld.shapes[5]


@given("a shape of known position and size")
def given_a_shape_of_known_position_and_size(context):
    prs = Presentation(test_pptx("shp-pos-and-size"))
    context.shape = prs.slides[0].shapes[0]


# when ====================================================


@when("I add a {cx} x {cy} shape at ({x}, {y})")
def when_I_add_a_cx_cy_shape_at_x_y(context, cx, cy, x, y):
    context.shape.shapes.add_shape(
        MSO_SHAPE.ROUNDED_RECTANGLE, int(x), int(y), int(cx), int(cy)
    )


@when("I assign 0.15 to shape.adjustments[0]")
def when_I_assign_to_shape_adjustments(context):
    context.shape.adjustments[0] = 0.15


@when("I assign builder.convert_to_shape() to shape")
def when_I_assign_builder_convert_to_shape_to_shape(context):
    builder = context.builder
    context.shape = builder.convert_to_shape()


@when("I assign builder.convert_to_shape({x_str}, {y_str}) to shape")
def when_I_assign_builder_convert_to_shape_origin_x_y(context, x_str, y_str):
    builder = context.builder
    origin_x, origin_y = int(x_str), int(y_str)
    context.shape = builder.convert_to_shape(origin_x, origin_y)


@when("I assign shape.text = {value}")
def when_I_assign_shape_text_eq_value(context, value):
    context.shape.text = eval(value)


@when("I assign {value} to connector.begin_x")
def when_I_assign_value_to_connector_begin_x(context, value):
    context.connector.begin_x = int(value)


@when("I assign {value} to connector.begin_y")
def when_I_assign_value_to_connector_begin_y(context, value):
    context.connector.begin_y = int(value)


@when("I assign {value} to connector.end_x")
def when_I_assign_value_to_connector_end_x(context, value):
    context.connector.end_x = int(value)


@when("I assign {value} to connector.end_y")
def when_I_assign_value_to_connector_end_y(context, value):
    context.connector.end_y = int(value)


@when("I assign {value} to picture.crop_{side}")
def when_I_assign_value_to_picture_crop_side(context, value, side):
    new_value = (
        None if value == "None" else float(value) if "." in value else int(value)
    )
    setattr(context.picture, "crop_%s" % side, new_value)


@when("I assign {value} to shape.height")
def when_I_assign_value_to_shape_height(context, value):
    context.shape.height = int(value)


@when("I assign {value} to shape.left")
def when_I_assign_value_to_shape_left(context, value):
    context.shape.left = int(value)


@when("I assign '{value}' to shape.name")
def when_I_assign_value_to_shape_name(context, value):
    context.shape.name = value


@when("I assign {value} to shape.rotation")
def when_I_assign_value_to_shape_rotation(context, value):
    context.shape.rotation = float(value)


@when("I assign {value} to shape.top")
def when_I_assign_value_to_shape_top(context, value):
    context.shape.top = int(value)


@when("I assign {value} to shape.width")
def when_I_assign_value_to_shape_width(context, value):
    context.shape.width = int(value)


@when("I call builder.add_line_segments([(100, 25), (25, 100)])")
def when_I_call_builder_add_line_segments_100_25_25_100(context):
    builder = context.builder
    builder.add_line_segments([(100, 25), (25, 100)])


@when("I call connector.begin_connect(picture, 3)")
def when_I_call_connector_begin_connect_picture_3(context):
    connector, picture = context.connector, context.picture
    connector.begin_connect(picture, 3)


@when("I call connector.end_connect(picture, 3)")
def when_I_call_connector_end_connect_picture_3(context):
    connector, picture = context.connector, context.picture
    connector.end_connect(picture, 3)


# then ====================================================


@then("accessing shape.click_action raises TypeError")
def then_accessing_shape_click_action_raises_TypeError(context):
    try:
        context.shape.click_action
    except TypeError:
        return
    except Exception as e:
        raise AssertionError(
            "Accessing GroupShape.click_action raised %s" % type(e).__name__
        )
    raise AssertionError("Accessing GroupShape.click_action did not raise")


@then("builder is a FreeformBuilder object")
def then_builder_is_a_FreeformBuilder_object(context):
    builder = context.builder
    class_name = builder.__class__.__name__
    expected_value = "FreeformBuilder"
    assert class_name == expected_value, "Expected class name '%s', got '%s'" % (
        expected_value,
        class_name,
    )


@then("(builder._start_x, builder._start_y) is ({x_str}, {y_str})")
def then_builder_start_x_builder_start_y_is_x_y(context, x_str, y_str):
    builder = context.builder
    actual_value = builder._start_x, builder._start_y
    expected_value = int(x_str), int(y_str)
    assert actual_value == expected_value, "Expected %s, got %s" % (
        expected_value,
        actual_value,
    )


@then("(builder._x_scale, builder._y_scale) is ({p_str}, {q_str})")
def then_builder_x_scale_builder_y_scale_is_x_y(context, p_str, q_str):
    builder = context.builder
    actual_value = builder._x_scale, builder._y_scale
    expected_value = float(p_str), float(q_str)
    assert actual_value == expected_value, "Expected %s, got %s" % (
        expected_value,
        actual_value,
    )


@then("connector is a Connector object")
def then_connector_is_a_Connector_object(context):
    assert type(context.connector).__name__ == "Connector"


@then("connector.begin_x == {value}")
def then_connector_begin_x_equals_value(context, value):
    assert context.connector.begin_x == int(value)


@then("connector.begin_x is an Emu object with value {x}")
def then_connector_begin_x_is_an_Emu_object_with_value_x(context, x):
    begin_x = context.connector.begin_x
    assert isinstance(begin_x, Emu)
    assert begin_x == int(x)


@then("connector.begin_y == {value}")
def then_connector_begin_y_equals_value(context, value):
    assert context.connector.begin_y == int(value)


@then("connector.begin_y is an Emu object with value {y}")
def then_connector_begin_y_is_an_Emu_object_with_value_y(context, y):
    begin_y = context.connector.begin_y
    assert isinstance(begin_y, Emu)
    assert begin_y == int(y)


@then("connector.end_x == {value}")
def then_connector_end_x_equals_value(context, value):
    assert context.connector.end_x == int(value)


@then("connector.end_x is an Emu object with value {x}")
def then_connector_end_x_is_an_Emu_object_with_value_x(context, x):
    end_x = context.connector.end_x
    assert isinstance(end_x, Emu)
    assert end_x == int(x)


@then("connector.end_y == {value}")
def then_connector_end_y_equals_value(context, value):
    assert context.connector.end_y == int(value)


@then("connector.end_y is an Emu object with value {y}")
def then_connector_end_y_is_an_Emu_object_with_value_y(context, y):
    end_y = context.connector.end_y
    assert isinstance(end_y, Emu)
    assert end_y == int(y)


@then("group_shape.shapes is a GroupShapes object")
def then_group_shape_shapes_is_a_GroupShapes_object(context):
    class_name = context.group_shape.shapes.__class__.__name__
    assert class_name == "GroupShapes", "got %s" % class_name


@then("len(ole_format.blob) == {value}")
def then_len_ole_format_blob_eq_value(context, value):
    actual = len(context.ole_format.blob)
    assert actual == int(value)


@then("movie is a Movie object")
def then_movie_is_a_Movie_object(context):
    class_name = context.movie.__class__.__name__
    assert class_name == "Movie", "got %s" % class_name


@then("movie.left, movie.top == x, y")
def then_movie_left_movie_top_eq_x_y(context):
    movie = context.movie
    position = movie.left, movie.top
    assert position == (Emu(2590800), Emu(571500)), "got %s" % position


@then("movie.media_format is a _MediaFormat object")
def then_movie_media_format_is_a_MediaFormat_object(context):
    class_name = context.movie.media_format.__class__.__name__
    assert class_name == "_MediaFormat", "got %s" % class_name


@then("movie.media_type is PP_MEDIA_TYPE.MOVIE")
def then_movie_media_type_is_PP_MEDIA_TYPE_MOVIE(context):
    media_type = context.movie.media_type
    assert media_type == PP_MEDIA_TYPE.MOVIE, "got %s" % media_type


@then("movie.poster_frame is the same image as poster_frame")
def then_movie_poster_frame_is_the_same_image_as_poster_frame(context):
    actual_sha1 = context.movie.poster_frame.sha1
    with open(test_file("just-two-mice.png"), "rb") as f:
        expected_sha1 = hashlib.sha1(f.read()).hexdigest()
    assert actual_sha1 == expected_sha1, "not the same image"


@then("movie.shape_type is MSO_SHAPE_TYPE.MEDIA")
def then_movie_shape_type_is_MSO_SHAPE_TYPE_MEDIA(context):
    shape_type = context.movie.shape_type
    assert shape_type == MSO_SHAPE_TYPE.MEDIA, "got %s" % shape_type


@then("movie.width, movie.height == cx, cy")
def then_movie_width_movie_height_eq_cx_cy(context):
    movie = context.movie
    size = movie.width, movie.height
    assert size == (Emu(3962400), Emu(5715000)), "got %s" % size


@then("ole_format.blob matches ole_object_file byte-for-byte")
def then_ole_format_bytes_matches_ole_object_file_byte_for_byte(context):
    assert context.ole_format.blob == context.ole_object_file.getvalue()


@then('ole_format.prog_id == "{expected_value}"')
def then_ole_format_prog_id_eq_value(context, expected_value):
    actual_value = context.ole_format.prog_id
    assert actual_value == expected_value, "expected %r, got %r" % (
        expected_value,
        actual_value,
    )


@then("ole_format.show_as_icon is {value}")
def then_ole_format_show_as_icon_is_value(context, value):
    expected_value = {"True": True, "False": False}[value]
    actual_value = context.ole_format.show_as_icon
    assert actual_value == expected_value, "expected %r, got %r" % (
        expected_value,
        actual_value,
    )


@then("picture.crop_{side} == {value}")
def then_picture_crop_side_eq_value(context, side, value):
    expected_value = round(float(value), 5)
    actual_value = round(getattr(context.picture, "crop_%s" % side), 5)
    assert actual_value == expected_value, "picture.crop_%s == %s" % (
        side,
        actual_value,
    )


@then("picture.image is an Image object")
def then_picture_image_is_an_Image_object(context):
    class_name = context.picture.image.__class__.__name__
    assert class_name == "Image", "picture.image is a %s object" % class_name


@then("shape.adjustments[0] is 0.15")
def then_shape_adjustments_is_value(context):
    shape = context.shape
    assert shape.adjustments[0] == 0.15


@then("shape.chart is a Chart object")
def then_shape_chart_is_a_Chart_object(context):
    chart = context.shape.chart
    class_name = chart.__class__.__name__
    assert class_name == "Chart", "got %s" % class_name


@then("shape.click_action is an ActionSetting object")
def then_shape_click_action_is_an_ActionSetting_object(context):
    assert isinstance(context.shape.click_action, ActionSetting)


@then("shape.has_chart is {value}")
def then_shape_has_chart_is_value(context, value):
    expected_value = {"True": True, "False": False}[value]
    actual_value = context.shape.has_chart
    assert actual_value is expected_value, "shape.has_chart is %s" % actual_value


@then("shape.has_table is {value}")
def then_shape_has_table_is_value(context, value):
    expected_value = {"True": True, "False": False}[value]
    actual_value = context.shape.has_table
    assert actual_value is expected_value, "shape.has_table is %s" % actual_value


@then("shape.has_text_frame is {value_str}")
def then_shape_has_text_frame_is(context, value_str):
    expected_value = {"True": True, "False": False}[value_str]
    has_text_frame = context.shape.has_text_frame
    assert has_text_frame is expected_value, "got %s" % has_text_frame


@then("shape.height == {value}")
def then_shape_height_eq_value(context, value):
    expected_height = int(value)
    actual_height = context.shape.height
    assert actual_height == expected_height, "shape.height == %s" % actual_height


@then("shape.left == {value}")
def then_shape_left_eq_value(context, value):
    expected_left = int(value)
    actual_left = context.shape.left
    assert actual_left == expected_left, "shape.left == %s" % actual_left


@then("shape.line is a LineFormat object")
def then_shape_line_is_a_LineFormat_object(context):
    shape = context.shape
    line_format = shape.line
    line_format_cls_name = cls_qname(line_format)
    expected_cls_name = "pptx.dml.line.LineFormat"
    assert line_format_cls_name == expected_cls_name, "expected '%s', got '%s'" % (
        expected_cls_name,
        line_format_cls_name,
    )


@then("shape.name == '{expected_value}'")
def then_shape_name_eq_value(context, expected_value):
    shape = context.shape
    msg = "expected shape name '%s', got '%s'" % (shape.name, expected_value)
    assert shape.name == expected_value, msg


@then("shape.ole_format is an _OleFormat object")
def then_shape_ole_format_is_an_OleFormat_object(context):
    cls_name = type(context.shape.ole_format).__name__
    expected_cls_name = "_OleFormat"
    assert cls_name == expected_cls_name, "expected %r, got %r" % (
        expected_cls_name,
        cls_name,
    )


@then("shape.part is a SlidePart object")
def then_shape_part_is_a_SlidePart_object(context):
    cls_name = type(context.shape.part).__name__
    expected_cls_name = "SlidePart"
    assert cls_name == expected_cls_name, "expected '%s', got '%s'" % (
        expected_cls_name,
        cls_name,
    )


@then("shape.part is slide.part")
def then_shape_part_is_slide_part(context):
    assert context.shape.part is context.slide.part


@then("shape.rotation == {value}")
def then_shape_rotation_eq_value(context, value):
    shape = context.shape
    expected_value = float(value)
    assert shape.rotation == expected_value, "got %s" % expected_value


@then("shape.shadow is a ShadowFormat object")
def then_shape_shadow_is_a_ShadowFormat_object(context):
    cls_name = type(context.shape.shadow).__name__
    assert cls_name == "ShadowFormat", "shape.shadow is a '%s' object" % cls_name


@then("shape.shadow raises NotImplementedError")
def then_shape_shadow_raises_NotImplementedError(context):
    try:
        context.shape.shadow
    except NotImplementedError:
        return
    except Exception as e:
        raise AssertionError("shape.shadow raises %s" % type(e).__name__)
    raise AssertionError("shape.shadow did not raise")


@then("shape.shape_id == {value_str}")
def then_shape_shape_id_equals(context, value_str):
    expected_value = int(value_str)
    shape_id = context.shape.shape_id
    assert shape_id == expected_value, "got %s" % shape_id


@then("shape.shape_type == MSO_SHAPE_TYPE.{member_name}")
def then_shape_shape_type_is_MSO_SHAPE_TYPE_member(context, member_name):
    expected_shape_type = getattr(MSO_SHAPE_TYPE, member_name)
    actual_shape_type = context.shape.shape_type
    assert actual_shape_type == expected_shape_type, (
        "shape.shape_type == %s" % actual_shape_type
    )


@then("shape.text == {value}")
def then_shape_text_eq_value(context, value):
    actual, expected = context.shape.text, eval(value)
    assert actual == expected, 'shape.text == "%s"' % actual


@then("shape.top == {value}")
def then_shape_top_eq_value(context, value):
    expected_top = int(value)
    actual_top = context.shape.top
    assert actual_top == expected_top, "shape.top == %s" % actual_top


@then("shape.width == {value}")
def then_shape_width_eq_value(context, value):
    expected_width = int(value)
    actual_width = context.shape.width
    assert actual_width == expected_width, "shape.width == %s" % actual_width



# encoding: utf-8

"""Connector (line) shape and related objects.

A connector is a line shape having end-points that can be connected to other
objects (but not to other connectors). A connector can be straight, have
elbows, or can be curved.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from pptx.dml.line import LineFormat
from pptx.enum.shapes import MSO_SHAPE_TYPE
from pptx.shapes.base import BaseShape
from pptx.util import Emu, lazyproperty


class Connector(BaseShape):
    """Connector (line) shape.

    A connector is a linear shape having end-points that can be connected to
    other objects (but not to other connectors). A connector can be straight,
    have elbows, or can be curved.
    """

    def begin_connect(self, shape, cxn_pt_idx):
        """
        **EXPERIMENTAL** - *The current implementation only works properly
        with rectangular shapes, such as pictures and rectangles. Use with
        other shape types may cause unexpected visual alignment of the
        connected end-point and could lead to a load error if cxn_pt_idx
        exceeds the connection point count available on the connected shape.
        That said, a quick test should reveal what to expect when using this
        method with other shape types.*

        Connect the beginning of this connector to *shape* at the connection
        point specified by *cxn_pt_idx*. Each shape has zero or more
        connection points and they are identified by index, starting with 0.
        Generally, the first connection point of a shape is at the top center
        of its bounding box and numbering proceeds counter-clockwise from
        there. However this is only a convention and may vary, especially
        with non built-in shapes.
        """
        self._connect_begin_to(shape, cxn_pt_idx)
        self._move_begin_to_cxn(shape, cxn_pt_idx)

    @property
    def begin_x(self):
        """
        Return the X-position of the begin point of this connector, in
        English Metric Units (as a |Length| object).
        """
        cxnSp = self._element
        x, cx, flipH = cxnSp.x, cxnSp.cx, cxnSp.flipH
        begin_x = x + cx if flipH else x
        return Emu(begin_x)

    @begin_x.setter
    def begin_x(self, value):
        cxnSp = self._element
        x, cx, flipH, new_x = cxnSp.x, cxnSp.cx, cxnSp.flipH, int(value)

        if flipH:
            old_x = x + cx
            dx = abs(new_x - old_x)
            if new_x >= old_x:
                cxnSp.cx = cx + dx
            elif dx <= cx:
                cxnSp.cx = cx - dx
            else:
                cxnSp.flipH = False
                cxnSp.x = new_x
                cxnSp.cx = dx - cx
        else:
            dx = abs(new_x - x)
            if new_x <= x:
                cxnSp.x = new_x
                cxnSp.cx = cx + dx
            elif dx <= cx:
                cxnSp.x = new_x
                cxnSp.cx = cx - dx
            else:
                cxnSp.flipH = True
                cxnSp.x = x + cx
                cxnSp.cx = dx - cx

    @property
    def begin_y(self):
        """
        Return the Y-position of the begin point of this connector, in
        English Metric Units (as a |Length| object).
        """
        cxnSp = self._element
        y, cy, flipV = cxnSp.y, cxnSp.cy, cxnSp.flipV
        begin_y = y + cy if flipV else y
        return Emu(begin_y)

    @begin_y.setter
    def begin_y(self, value):
        cxnSp = self._element
        y, cy, flipV, new_y = cxnSp.y, cxnSp.cy, cxnSp.flipV, int(value)

        if flipV:
            old_y = y + cy
            dy = abs(new_y - old_y)
            if new_y >= old_y:
                cxnSp.cy = cy + dy
            elif dy <= cy:
                cxnSp.cy = cy - dy
            else:
                cxnSp.flipV = False
                cxnSp.y = new_y
                cxnSp.cy = dy - cy
        else:
            dy = abs(new_y - y)
            if new_y <= y:
                cxnSp.y = new_y
                cxnSp.cy = cy + dy
            elif dy <= cy:
                cxnSp.y = new_y
                cxnSp.cy = cy - dy
            else:
                cxnSp.flipV = True
                cxnSp.y = y + cy
                cxnSp.cy = dy - cy

    def end_connect(self, shape, cxn_pt_idx):
        """
        **EXPERIMENTAL** - *The current implementation only works properly
        with rectangular shapes, such as pictures and rectangles. Use with
        other shape types may cause unexpected visual alignment of the
        connected end-point and could lead to a load error if cxn_pt_idx
        exceeds the connection point count available on the connected shape.
        That said, a quick test should reveal what to expect when using this
        method with other shape types.*

        Connect the ending of this connector to *shape* at the connection
        point specified by *cxn_pt_idx*.
        """
        self._connect_end_to(shape, cxn_pt_idx)
        self._move_end_to_cxn(shape, cxn_pt_idx)

    @property
    def end_x(self):
        """
        Return the X-position of the end point of this connector, in English
        Metric Units (as a |Length| object).
        """
        cxnSp = self._element
        x, cx, flipH = cxnSp.x, cxnSp.cx, cxnSp.flipH
        end_x = x if flipH else x + cx
        return Emu(end_x)

    @end_x.setter
    def end_x(self, value):
        cxnSp = self._element
        x, cx, flipH, new_x = cxnSp.x, cxnSp.cx, cxnSp.flipH, int(value)

        if flipH:
            dx = abs(new_x - x)
            if new_x <= x:
                cxnSp.x = new_x
                cxnSp.cx = cx + dx
            elif dx <= cx:
                cxnSp.x = new_x
                cxnSp.cx = cx - dx
            else:
                cxnSp.flipH = False
                cxnSp.x = x + cx
                cxnSp.cx = dx - cx
        else:
            old_x = x + cx
            dx = abs(new_x - old_x)
            if new_x >= old_x:
                cxnSp.cx = cx + dx
            elif dx <= cx:
                cxnSp.cx = cx - dx
            else:
                cxnSp.flipH = True
                cxnSp.x = new_x
                cxnSp.cx = dx - cx

    @property
    def end_y(self):
        """
        Return the Y-position of the end point of this connector, in English
        Metric Units (as a |Length| object).
        """
        cxnSp = self._element
        y, cy, flipV = cxnSp.y, cxnSp.cy, cxnSp.flipV
        end_y = y if flipV else y + cy
        return Emu(end_y)

    @end_y.setter
    def end_y(self, value):
        cxnSp = self._element
        y, cy, flipV, new_y = cxnSp.y, cxnSp.cy, cxnSp.flipV, int(value)

        if flipV:
            dy = abs(new_y - y)
            if new_y <= y:
                cxnSp.y = new_y
                cxnSp.cy = cy + dy
            elif dy <= cy:
                cxnSp.y = new_y
                cxnSp.cy = cy - dy
            else:
                cxnSp.flipV = False
                cxnSp.y = y + cy
                cxnSp.cy = dy - cy
        else:
            old_y = y + cy
            dy = abs(new_y - old_y)
            if new_y >= old_y:
                cxnSp.cy = cy + dy
            elif dy <= cy:
                cxnSp.cy = cy - dy
            else:
                cxnSp.flipV = True
                cxnSp.y = new_y
                cxnSp.cy = dy - cy

    def get_or_add_ln(self):
        """Helper method required by |LineFormat|."""
        return self._element.spPr.get_or_add_ln()

    @lazyproperty
    def line(self):
        """|LineFormat| instance for this connector.

        Provides access to line properties such as line color, width, and
        line style.
        """
        return LineFormat(self)

    @property
    def ln(self):
        """Helper method required by |LineFormat|.

        The ``<a:ln>`` element containing the line format properties such as
        line color and width. |None| if no `<a:ln>` element is present.
        """
        return self._element.spPr.ln

    @property
    def shape_type(self):
        """Member of `MSO_SHAPE_TYPE` identifying the type of this shape.

        Unconditionally `MSO_SHAPE_TYPE.LINE` for a `Connector` object.
        """
        return MSO_SHAPE_TYPE.LINE

    def _connect_begin_to(self, shape, cxn_pt_idx):
        """
        Add or update a stCxn element for this connector that connects its
        begin point to the connection point of *shape* specified by
        *cxn_pt_idx*.
        """
        cNvCxnSpPr = self._element.nvCxnSpPr.cNvCxnSpPr
        stCxn = cNvCxnSpPr.get_or_add_stCxn()
        stCxn.id = shape.shape_id
        stCxn.idx = cxn_pt_idx

    def _connect_end_to(self, shape, cxn_pt_idx):
        """
        Add or update an endCxn element for this connector that connects its
        end point to the connection point of *shape* specified by
        *cxn_pt_idx*.
        """
        cNvCxnSpPr = self._element.nvCxnSpPr.cNvCxnSpPr
        endCxn = cNvCxnSpPr.get_or_add_endCxn()
        endCxn.id = shape.shape_id
        endCxn.idx = cxn_pt_idx

    def _move_begin_to_cxn(self, shape, cxn_pt_idx):
        """
        Move the begin point of this connector to coordinates of the
        connection point of *shape* specified by *cxn_pt_idx*.
        """
        x, y, cx, cy = shape.left, shape.top, shape.width, shape.height
        self.begin_x, self.begin_y = {
            0: (int(x + cx / 2), y),
            1: (x, int(y + cy / 2)),
            2: (int(x + cx / 2), y + cy),
            3: (x + cx, int(y + cy / 2)),
        }[cxn_pt_idx]

    def _move_end_to_cxn(self, shape, cxn_pt_idx):
        """
        Move the end point of this connector to the coordinates of the
        connection point of *shape* specified by *cxn_pt_idx*.
        """
        x, y, cx, cy = shape.left, shape.top, shape.width, shape.height
        self.end_x, self.end_y = {
            0: (int(x + cx / 2), y),
            1: (x, int(y + cy / 2)),
            2: (int(x + cx / 2), y + cy),
            3: (x + cx, int(y + cy / 2)),
        }[cxn_pt_idx]



# encoding: utf-8

"""Gherkin step implementations for slide collection-related features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, when, then

from pptx import Presentation

from helpers import test_pptx


# given ===================================================


@given("a SlideLayouts object containing 2 layouts as slide_layouts")
def given_a_SlideLayouts_object_containing_2_layouts(context):
    prs = Presentation(test_pptx("mst-slide-layouts"))
    context.slide_layouts = prs.slide_master.slide_layouts


@given("a SlideMasters object containing 2 masters")
def given_a_SlideMasters_object_containing_2_masters(context):
    prs = Presentation(test_pptx("prs-slide-masters"))
    context.slide_masters = prs.slide_masters


@given("a Slides object containing 3 slides")
def given_a_Slides_object_containing_3_slides(context):
    prs = Presentation(test_pptx("sld-slides"))
    context.prs = prs
    context.slides = prs.slides


# when ====================================================


@when("I call slides.add_slide()")
def when_I_call_slides_add_slide(context):
    context.slide_layout = context.prs.slide_masters[0].slide_layouts[0]
    context.slides.add_slide(context.slide_layout)


@when("I call slide_layouts.remove(slide_layouts[1])")
def when_I_call_slide_layouts_remove(context):
    slide_layouts = context.slide_layouts
    slide_layouts.remove(slide_layouts[1])


# then ====================================================


@then("iterating produces 3 NotesSlidePlaceholder objects")
def then_iterating_produces_3_NotesSlidePlaceholder_objects(context):
    idx = -1
    for idx, placeholder in enumerate(context.notes_slide.placeholders):
        typename = type(placeholder).__name__
        assert typename == "NotesSlidePlaceholder", "got %s" % typename
    assert idx == 2


@then("iterating slide_layouts produces 2 SlideLayout objects")
def then_iterating_slide_layouts_produces_2_SlideLayout_objects(context):
    slide_layouts = context.slide_layouts
    idx = -1
    for idx, slide_layout in enumerate(slide_layouts):
        assert type(slide_layout).__name__ == "SlideLayout"
    assert idx == 1


@then("iterating slide_masters produces 2 SlideMaster objects")
def then_iterating_slide_masters_produces_2_SlideMaster_objects(context):
    slide_masters = context.slide_masters
    idx = -1
    for idx, slide_master in enumerate(slide_masters):
        assert type(slide_master).__name__ == "SlideMaster"
    assert idx == 1


@then("iterating slides produces 3 Slide objects")
def then_iterating_slides_produces_3_Slide_objects(context):
    slides = context.slides
    idx = -1
    for idx, slide in enumerate(slides):
        assert type(slide).__name__ == "Slide"
    assert idx == 2


@then("len(slides) is {count}")
def then_len_slides_is_count(context, count):
    slides = context.slides
    assert len(slides) == int(count)


@then("len(slide_layouts) is {n}")
def then_len_slide_layouts_is_2(context, n):
    assert len(context.slide_layouts) == int(n)


@then("len(slide_masters) is 2")
def then_len_slide_masters_is_2(context):
    slide_masters = context.slide_masters
    assert len(slide_masters) == 2


@then("slide_layouts[1] is a SlideLayout object")
def then_slide_layouts_1_is_a_SlideLayout_object(context):
    slide_layouts = context.slide_layouts
    assert type(slide_layouts[1]).__name__ == "SlideLayout"


@then("slide_layouts.get_by_name(slide_layouts[1].name) is slide_layouts[1]")
def then_slide_layouts_get_by_name_is_slide_layout(context):
    slide_layouts = context.slide_layouts
    assert slide_layouts.get_by_name(slide_layouts[1].name) is slide_layouts[1]


@then("slide_layouts.index(slide_layouts[1]) == 1")
def then_slide_layouts_index_is_1(context):
    slide_layouts = context.slide_layouts
    assert slide_layouts.index(slide_layouts[1]) == 1


@then("slide_masters[1] is a SlideMaster object")
def then_slide_masters_1_is_a_SlideMaster_object(context):
    slide_masters = context.slide_masters
    assert type(slide_masters[1]).__name__ == "SlideMaster"


@then("slides.get(256) is slides[0]")
def then_slides_get_256_is_slides_0(context):
    slides = context.slides
    assert slides.get(256) is slides[0]


@then("slides.get(666, default=slides[2]) is slides[2]")
def then_slides_get_666_default_slides_2_is_slides_2(context):
    slides = context.slides
    assert slides.get(666, default=slides[2]) is slides[2]


@then("slides[2] is a Slide object")
def then_slides_2_is_a_Slide_object(context):
    slides = context.slides
    assert type(slides[2]).__name__ == "Slide"



# encoding: utf-8

"""Custom element classes for top-level chart-related XML elements."""

from __future__ import absolute_import, division, print_function, unicode_literals

from pptx.oxml import parse_xml
from pptx.oxml.chart.shared import CT_Title
from pptx.oxml.ns import nsdecls, qn
from pptx.oxml.simpletypes import ST_Style, XsdString
from pptx.oxml.text import CT_TextBody
from pptx.oxml.xmlchemy import (
    BaseOxmlElement,
    OneAndOnlyOne,
    RequiredAttribute,
    ZeroOrMore,
    ZeroOrOne,
)


class CT_Chart(BaseOxmlElement):
    """`c:chart` custom element class."""

    _tag_seq = (
        "c:title",
        "c:autoTitleDeleted",
        "c:pivotFmts",
        "c:view3D",
        "c:floor",
        "c:sideWall",
        "c:backWall",
        "c:plotArea",
        "c:legend",
        "c:plotVisOnly",
        "c:dispBlanksAs",
        "c:showDLblsOverMax",
        "c:extLst",
    )
    title = ZeroOrOne("c:title", successors=_tag_seq[1:])
    autoTitleDeleted = ZeroOrOne("c:autoTitleDeleted", successors=_tag_seq[2:])
    plotArea = OneAndOnlyOne("c:plotArea")
    legend = ZeroOrOne("c:legend", successors=_tag_seq[9:])
    rId = RequiredAttribute("r:id", XsdString)

    _chart_tmpl = '<c:chart %s %s r:id="%%s"/>' % (nsdecls("c"), nsdecls("r"))

    @property
    def has_legend(self):
        """
        True if this chart has a legend defined, False otherwise.
        """
        legend = self.legend
        if legend is None:
            return False
        return True

    @has_legend.setter
    def has_legend(self, bool_value):
        """
        Add, remove, or leave alone the ``<c:legend>`` child element depending
        on current state and *bool_value*. If *bool_value* is |True| and no
        ``<c:legend>`` element is present, a new default element is added.
        When |False|, any existing legend element is removed.
        """
        if bool(bool_value) is False:
            self._remove_legend()
        else:
            if self.legend is None:
                self._add_legend()

    @staticmethod
    def new_chart(rId):
        """
        Return a new ``<c:chart>`` element
        """
        xml = CT_Chart._chart_tmpl % (rId)
        chart = parse_xml(xml)
        return chart

    def _new_title(self):
        return CT_Title.new_title()


class CT_ChartSpace(BaseOxmlElement):
    """`c:chartSpace` root element of a chart part."""

    _tag_seq = (
        "c:date1904",
        "c:lang",
        "c:roundedCorners",
        "c:style",
        "c:clrMapOvr",
        "c:pivotSource",
        "c:protection",
        "c:chart",
        "c:spPr",
        "c:txPr",
        "c:externalData",
        "c:printSettings",
        "c:userShapes",
        "c:extLst",
    )
    date1904 = ZeroOrOne("c:date1904", successors=_tag_seq[1:])
    style = ZeroOrOne("c:style", successors=_tag_seq[4:])
    chart = OneAndOnlyOne("c:chart")
    txPr = ZeroOrOne("c:txPr", successors=_tag_seq[10:])
    externalData = ZeroOrOne("c:externalData", successors=_tag_seq[11:])
    del _tag_seq

    @property
    def catAx_lst(self):
        return self.chart.plotArea.catAx_lst

    @property
    def date_1904(self):
        """
        Return |True| if the `c:date1904` child element resolves truthy,
        |False| otherwise. This value indicates whether date number values
        are based on the 1900 or 1904 epoch.
        """
        date1904 = self.date1904
        if date1904 is None:
            return False
        return date1904.val

    @property
    def dateAx_lst(self):
        return self.xpath("c:chart/c:plotArea/c:dateAx")

    def get_or_add_title(self):
        """Return the `c:title` grandchild, newly created if not present."""
        return self.chart.get_or_add_title()

    @property
    def plotArea(self):
        """
        Return the required `c:chartSpace/c:chart/c:plotArea` grandchild
        element.
        """
        return self.chart.plotArea

    @property
    def valAx_lst(self):
        return self.chart.plotArea.valAx_lst

    @property
    def xlsx_part_rId(self):
        """
        The string in the required ``r:id`` attribute of the
        `<c:externalData>` child, or |None| if no externalData element is
        present.
        """
        externalData = self.externalData
        if externalData is None:
            return None
        return externalData.rId

    def _add_externalData(self):
        """
        Always add a ``<c:autoUpdate val="0"/>`` child so auto-updating
        behavior is off by default.
        """
        externalData = self._new_externalData()
        externalData._add_autoUpdate(val=False)
        self._insert_externalData(externalData)
        return externalData

    def _new_txPr(self):
        return CT_TextBody.new_txPr()


class CT_ExternalData(BaseOxmlElement):
    """
    `<c:externalData>` element, defining link to embedded Excel package part
    containing the chart data.
    """

    autoUpdate = ZeroOrOne("c:autoUpdate")
    rId = RequiredAttribute("r:id", XsdString)


class CT_PlotArea(BaseOxmlElement):
    """
    ``<c:plotArea>`` element.
    """

    catAx = ZeroOrMore("c:catAx")
    valAx = ZeroOrMore("c:valAx")

    def iter_sers(self):
        """
        Generate each of the `c:ser` elements in this chart, ordered first by
        the document order of the containing xChart element, then by their
        ordering within the xChart element (not necessarily document order).
        """
        for xChart in self.iter_xCharts():
            for ser in xChart.iter_sers():
                yield ser

    def iter_xCharts(self):
        """
        Generate each xChart child element in document.
        """
        plot_tags = (
            qn("c:area3DChart"),
            qn("c:areaChart"),
            qn("c:bar3DChart"),
            qn("c:barChart"),
            qn("c:bubbleChart"),
            qn("c:doughnutChart"),
            qn("c:line3DChart"),
            qn("c:lineChart"),
            qn("c:ofPieChart"),
            qn("c:pie3DChart"),
            qn("c:pieChart"),
            qn("c:radarChart"),
            qn("c:scatterChart"),
            qn("c:stockChart"),
            qn("c:surface3DChart"),
            qn("c:surfaceChart"),
        )

        for child in self.iterchildren():
            if child.tag not in plot_tags:
                continue
            yield child

    @property
    def last_ser(self):
        """
        Return the last `<c:ser>` element in the last xChart element, based
        on series order (not necessarily the same element as document order).
        """
        last_xChart = self.xCharts[-1]
        sers = last_xChart.sers
        if not sers:
            return None
        return sers[-1]

    @property
    def next_idx(self):
        """
        Return the next available `c:ser/c:idx` value within the scope of
        this chart, the maximum idx value found on existing series,
        incremented by one.
        """
        idx_vals = [s.idx.val for s in self.sers]
        if not idx_vals:
            return 0
        return max(idx_vals) + 1

    @property
    def next_order(self):
        """
        Return the next available `c:ser/c:order` value within the scope of
        this chart, the maximum order value found on existing series,
        incremented by one.
        """
        order_vals = [s.order.val for s in self.sers]
        if not order_vals:
            return 0
        return max(order_vals) + 1

    @property
    def sers(self):
        """
        Return a sequence containing all the `c:ser` elements in this chart,
        ordered first by the document order of the containing xChart element,
        then by their ordering within the xChart element (not necessarily
        document order).
        """
        return tuple(self.iter_sers())

    @property
    def xCharts(self):
        """
        Return a sequence containing all the `c:{x}Chart` elements in this
        chart, in document order.
        """
        return tuple(self.iter_xCharts())


class CT_Style(BaseOxmlElement):
    """
    ``<c:style>`` element; defines the chart style.
    """

    val = RequiredAttribute("val", ST_Style)



# encoding: utf-8

"""
Test suite for the pptx.oxml.xmlchemy module, focused on the metaclass and
element and attribute definition classes. A major part of the fixture is
provided by the metaclass-built test classes at the end of the file.
"""

from __future__ import absolute_import, print_function

import pytest

from pptx.exc import InvalidXmlError
from pptx.oxml import register_element_cls
from pptx.oxml.ns import qn
from pptx.oxml.simpletypes import BaseIntType
from pptx.oxml.xmlchemy import (
    BaseOxmlElement,
    Choice,
    OneAndOnlyOne,
    OneOrMore,
    OptionalAttribute,
    RequiredAttribute,
    ZeroOrMore,
    ZeroOrOne,
    ZeroOrOneChoice,
)

from ..unitdata import BaseBuilder


class DescribeCustomElementClass(object):
    def it_has_the_MetaOxmlElement_metaclass(self):
        assert type(CT_Parent).__name__ == "MetaOxmlElement"


class DescribeChoice(object):
    def it_adds_a_getter_property_for_the_choice_element(self, getter_fixture):
        parent, expected_choice = getter_fixture
        assert parent.choice is expected_choice

    def it_adds_a_creator_method_for_the_child_element(self, new_fixture):
        parent, expected_xml = new_fixture
        choice = parent._new_choice()
        assert choice.xml == expected_xml

    def it_adds_an_insert_method_for_the_child_element(self, insert_fixture):
        parent, choice, expected_xml = insert_fixture
        parent._insert_choice(choice)
        assert parent.xml == expected_xml
        assert parent._insert_choice.__doc__.startswith(
            "Return the passed ``<p:choice>`` "
        )

    def it_adds_an_add_method_for_the_child_element(self, add_fixture):
        parent, expected_xml = add_fixture
        choice = parent._add_choice()
        assert parent.xml == expected_xml
        assert isinstance(choice, CT_Choice)
        assert parent._add_choice.__doc__.startswith(
            "Add a new ``<p:choice>`` child element "
        )

    def it_adds_a_get_or_change_to_method_for_the_child_element(
        self, get_or_change_to_fixture
    ):
        parent, expected_xml = get_or_change_to_fixture
        choice = parent.get_or_change_to_choice()
        assert isinstance(choice, CT_Choice)
        assert parent.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_fixture(self):
        parent = self.parent_bldr().element
        expected_xml = self.parent_bldr("choice").xml()
        return parent, expected_xml

    @pytest.fixture(
        params=[("choice2", "choice"), (None, "choice"), ("choice", "choice")]
    )
    def get_or_change_to_fixture(self, request):
        before_member_tag, after_member_tag = request.param
        parent = self.parent_bldr(before_member_tag).element
        expected_xml = self.parent_bldr(after_member_tag).xml()
        return parent, expected_xml

    @pytest.fixture(params=["choice", None])
    def getter_fixture(self, request):
        choice_tag = request.param
        parent = self.parent_bldr(choice_tag).element
        expected_choice = parent.find(qn("p:choice"))  # None if not found
        return parent, expected_choice

    @pytest.fixture
    def insert_fixture(self):
        parent = (
            a_parent()
            .with_nsdecls()
            .with_child(an_oomChild())
            .with_child(an_oooChild())
        ).element
        choice = a_choice().with_nsdecls().element
        expected_xml = (
            a_parent()
            .with_nsdecls()
            .with_child(a_choice())
            .with_child(an_oomChild())
            .with_child(an_oooChild())
        ).xml()
        return parent, choice, expected_xml

    @pytest.fixture
    def new_fixture(self):
        parent = self.parent_bldr().element
        expected_xml = a_choice().with_nsdecls().xml()
        return parent, expected_xml

    # fixture components ---------------------------------------------

    def parent_bldr(self, choice_tag=None):
        parent_bldr = a_parent().with_nsdecls()
        if choice_tag == "choice":
            parent_bldr.with_child(a_choice())
        if choice_tag == "choice2":
            parent_bldr.with_child(a_choice2())
        return parent_bldr


class DescribeOneAndOnlyOne(object):
    def it_adds_a_getter_property_for_the_child_element(self, getter_fixture):
        parent, oooChild = getter_fixture
        assert parent.oooChild is oooChild

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def getter_fixture(self):
        parent = a_parent().with_nsdecls().with_child(an_oooChild()).element
        oooChild = parent.find(qn("p:oooChild"))
        return parent, oooChild


class DescribeOneOrMore(object):
    def it_adds_a_getter_property_for_the_child_element_list(self, getter_fixture):
        parent, oomChild = getter_fixture
        assert parent.oomChild_lst[0] is oomChild

    def it_adds_a_creator_method_for_the_child_element(self, new_fixture):
        parent, expected_xml = new_fixture
        oomChild = parent._new_oomChild()
        assert oomChild.xml == expected_xml

    def it_adds_an_insert_method_for_the_child_element(self, insert_fixture):
        parent, oomChild, expected_xml = insert_fixture
        parent._insert_oomChild(oomChild)
        assert parent.xml == expected_xml
        assert parent._insert_oomChild.__doc__.startswith(
            "Return the passed ``<p:oomChild>`` "
        )

    def it_adds_a_private_add_method_for_the_child_element(self, add_fixture):
        parent, expected_xml = add_fixture
        oomChild = parent._add_oomChild()
        assert parent.xml == expected_xml
        assert isinstance(oomChild, CT_OomChild)
        assert parent._add_oomChild.__doc__.startswith(
            "Add a new ``<p:oomChild>`` child element "
        )

    def it_adds_a_public_add_method_for_the_child_element(self, add_fixture):
        parent, expected_xml = add_fixture
        oomChild = parent.add_oomChild()
        assert parent.xml == expected_xml
        assert isinstance(oomChild, CT_OomChild)
        assert parent._add_oomChild.__doc__.startswith(
            "Add a new ``<p:oomChild>`` child element "
        )

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_fixture(self):
        parent = self.parent_bldr(False).element
        expected_xml = self.parent_bldr(True).xml()
        return parent, expected_xml

    @pytest.fixture
    def getter_fixture(self):
        parent = self.parent_bldr(True).element
        oomChild = parent.find(qn("p:oomChild"))
        return parent, oomChild

    @pytest.fixture
    def insert_fixture(self):
        parent = (
            a_parent()
            .with_nsdecls()
            .with_child(an_oooChild())
            .with_child(a_zomChild())
            .with_child(a_zooChild())
        ).element
        oomChild = an_oomChild().with_nsdecls().element
        expected_xml = (
            a_parent()
            .with_nsdecls()
            .with_child(an_oomChild())
            .with_child(an_oooChild())
            .with_child(a_zomChild())
            .with_child(a_zooChild())
        ).xml()
        return parent, oomChild, expected_xml

    @pytest.fixture
    def new_fixture(self):
        parent = self.parent_bldr(False).element
        expected_xml = an_oomChild().with_nsdecls().xml()
        return parent, expected_xml

    # fixture components ---------------------------------------------

    def parent_bldr(self, oomChild_is_present):
        parent_bldr = a_parent().with_nsdecls()
        if oomChild_is_present:
            parent_bldr.with_child(an_oomChild())
        return parent_bldr


class DescribeOptionalAttribute(object):
    def it_adds_a_getter_property_for_the_attr_value(self, getter_fixture):
        parent, optAttr_python_value = getter_fixture
        assert parent.optAttr == optAttr_python_value

    def it_adds_a_setter_property_for_the_attr(self, setter_fixture):
        parent, value, expected_xml = setter_fixture
        parent.optAttr = value
        assert parent.xml == expected_xml

    def it_adds_a_docstring_for_the_property(self):
        assert CT_Parent.optAttr.__doc__.startswith(
            "ST_IntegerType type-converted value of "
        )

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def getter_fixture(self):
        parent = a_parent().with_nsdecls().with_optAttr("24").element
        return parent, 24

    @pytest.fixture(params=[36, None])
    def setter_fixture(self, request):
        value = request.param
        parent = a_parent().with_nsdecls().with_optAttr("42").element
        if value is None:
            expected_xml = a_parent().with_nsdecls().xml()
        else:
            expected_xml = a_parent().with_nsdecls().with_optAttr(value).xml()
        return parent, value, expected_xml


class DescribeRequiredAttribute(object):
    def it_adds_a_getter_property_for_the_attr_value(self, getter_fixture):
        parent, reqAttr_python_value = getter_fixture
        assert parent.reqAttr == reqAttr_python_value

    def it_adds_a_setter_property_for_the_attr(self, setter_fixture):
        parent, value, expected_xml = setter_fixture
        parent.reqAttr = value
        assert parent.xml == expected_xml

    def it_adds_a_docstring_for_the_property(self):
        assert CT_Parent.reqAttr.__doc__.startswith(
            "ST_IntegerType type-converted value of "
        )

    def it_raises_on_get_when_attribute_not_present(self):
        parent = a_parent().with_nsdecls().element
        with pytest.raises(InvalidXmlError):
            parent.reqAttr

    def it_raises_on_assign_invalid_value(self, invalid_assign_fixture):
        parent, value, expected_exception = invalid_assign_fixture
        with pytest.raises(expected_exception):
            parent.reqAttr = value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def getter_fixture(self):
        parent = a_parent().with_nsdecls().with_reqAttr("42").element
        return parent, 42

    @pytest.fixture(params=[(None, TypeError), (-4, ValueError), ("2", TypeError)])
    def invalid_assign_fixture(self, request):
        invalid_value, expected_exception = request.param
        parent = a_parent().with_nsdecls().with_reqAttr(1).element
        return parent, invalid_value, expected_exception

    @pytest.fixture
    def setter_fixture(self):
        parent = a_parent().with_nsdecls().with_reqAttr("42").element
        value = 24
        expected_xml = a_parent().with_nsdecls().with_reqAttr(value).xml()
        return parent, value, expected_xml


class DescribeZeroOrMore(object):
    def it_adds_a_getter_property_for_the_child_element_list(self, getter_fixture):
        parent, zomChild = getter_fixture
        assert parent.zomChild_lst[0] is zomChild

    def it_adds_a_creator_method_for_the_child_element(self, new_fixture):
        parent, expected_xml = new_fixture
        zomChild = parent._new_zomChild()
        assert zomChild.xml == expected_xml

    def it_adds_an_insert_method_for_the_child_element(self, insert_fixture):
        parent, zomChild, expected_xml = insert_fixture
        parent._insert_zomChild(zomChild)
        assert parent.xml == expected_xml
        assert parent._insert_zomChild.__doc__.startswith(
            "Return the passed ``<p:zomChild>`` "
        )

    def it_adds_an_add_method_for_the_child_element(self, add_fixture):
        parent, expected_xml = add_fixture
        zomChild = parent._add_zomChild()
        assert parent.xml == expected_xml
        assert isinstance(zomChild, CT_ZomChild)
        assert parent._add_zomChild.__doc__.startswith(
            "Add a new ``<p:zomChild>`` child element "
        )

    def it_removes_the_property_root_name_used_for_declaration(self):
        assert not hasattr(CT_Parent, "zomChild")

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_fixture(self):
        parent = self.parent_bldr(False).element
        expected_xml = self.parent_bldr(True).xml()
        return parent, expected_xml

    @pytest.fixture
    def getter_fixture(self):
        parent = self.parent_bldr(True).element
        zomChild = parent.find(qn("p:zomChild"))
        return parent, zomChild

    @pytest.fixture
    def insert_fixture(self):
        parent = (
            a_parent()
            .with_nsdecls()
            .with_child(an_oomChild())
            .with_child(an_oooChild())
            .with_child(a_zooChild())
        ).element
        zomChild = a_zomChild().with_nsdecls().element
        expected_xml = (
            a_parent()
            .with_nsdecls()
            .with_child(an_oomChild())
            .with_child(an_oooChild())
            .with_child(a_zomChild())
            .with_child(a_zooChild())
        ).xml()
        return parent, zomChild, expected_xml

    @pytest.fixture
    def new_fixture(self):
        parent = self.parent_bldr(False).element
        expected_xml = a_zomChild().with_nsdecls().xml()
        return parent, expected_xml

    def parent_bldr(self, zomChild_is_present):
        parent_bldr = a_parent().with_nsdecls()
        if zomChild_is_present:
            parent_bldr.with_child(a_zomChild())
        return parent_bldr


class DescribeZeroOrOne(object):
    def it_adds_a_getter_property_for_the_child_element(self, getter_fixture):
        parent, zooChild = getter_fixture
        assert parent.zooChild is zooChild

    def it_adds_an_add_method_for_the_child_element(self, add_fixture):
        parent, expected_xml = add_fixture
        zooChild = parent._add_zooChild()
        assert parent.xml == expected_xml
        assert isinstance(zooChild, CT_ZooChild)
        assert parent._add_zooChild.__doc__.startswith(
            "Add a new ``<p:zooChild>`` child element "
        )

    def it_adds_an_insert_method_for_the_child_element(self, insert_fixture):
        parent, zooChild, expected_xml = insert_fixture
        parent._insert_zooChild(zooChild)
        assert parent.xml == expected_xml
        assert parent._insert_zooChild.__doc__.startswith(
            "Return the passed ``<p:zooChild>`` "
        )

    def it_adds_a_get_or_add_method_for_the_child_element(self, get_or_add_fixture):
        parent, expected_xml = get_or_add_fixture
        zooChild = parent.get_or_add_zooChild()
        assert isinstance(zooChild, CT_ZooChild)
        assert parent.xml == expected_xml

    def it_adds_a_remover_method_for_the_child_element(self, remove_fixture):
        parent, expected_xml = remove_fixture
        parent._remove_zooChild()
        assert parent.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_fixture(self):
        parent = self.parent_bldr(False).element
        expected_xml = self.parent_bldr(True).xml()
        return parent, expected_xml

    @pytest.fixture(params=[True, False])
    def getter_fixture(self, request):
        zooChild_is_present = request.param
        parent = self.parent_bldr(zooChild_is_present).element
        zooChild = parent.find(qn("p:zooChild"))  # None if not found
        return parent, zooChild

    @pytest.fixture(params=[True, False])
    def get_or_add_fixture(self, request):
        zooChild_is_present = request.param
        parent = self.parent_bldr(zooChild_is_present).element
        expected_xml = self.parent_bldr(True).xml()
        return parent, expected_xml

    @pytest.fixture
    def insert_fixture(self):
        parent = (
            a_parent()
            .with_nsdecls()
            .with_child(an_oomChild())
            .with_child(an_oooChild())
            .with_child(a_zomChild())
        ).element
        zooChild = a_zooChild().with_nsdecls().element
        expected_xml = (
            a_parent()
            .with_nsdecls()
            .with_child(an_oomChild())
            .with_child(an_oooChild())
            .with_child(a_zomChild())
            .with_child(a_zooChild())
        ).xml()
        return parent, zooChild, expected_xml

    @pytest.fixture(params=[True, False])
    def remove_fixture(self, request):
        zooChild_is_present = request.param
        parent = self.parent_bldr(zooChild_is_present).element
        expected_xml = self.parent_bldr(False).xml()
        return parent, expected_xml

    # fixture components ---------------------------------------------

    def parent_bldr(self, zooChild_is_present):
        parent_bldr = a_parent().with_nsdecls()
        if zooChild_is_present:
            parent_bldr.with_child(a_zooChild())
        return parent_bldr


class DescribeZeroOrOneChoice(object):
    def it_adds_a_getter_for_the_current_choice(self, getter_fixture):
        parent, expected_choice = getter_fixture
        assert parent.eg_zooChoice is expected_choice

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[None, "choice", "choice2"])
    def getter_fixture(self, request):
        choice_tag = request.param
        parent = self.parent_bldr(choice_tag).element
        tagname = "p:%s" % choice_tag
        expected_choice = parent.find(qn(tagname))  # None if not found
        return parent, expected_choice

    # fixture components ---------------------------------------------

    def parent_bldr(self, choice_tag=None):
        parent_bldr = a_parent().with_nsdecls()
        if choice_tag == "choice":
            parent_bldr.with_child(a_choice())
        if choice_tag == "choice2":
            parent_bldr.with_child(a_choice2())
        return parent_bldr


# --------------------------------------------------------------------
# static shared fixture
# --------------------------------------------------------------------


class ST_IntegerType(BaseIntType):
    @classmethod
    def validate(cls, value):
        cls.validate_int(value)
        if value < 1 or value > 42:
            raise ValueError("value must be in range 1 to 42 inclusive")


class CT_Parent(BaseOxmlElement):
    """
    ``<p:parent>`` element, an invented element for use in testing.
    """

    eg_zooChoice = ZeroOrOneChoice(
        (Choice("p:choice"), Choice("p:choice2")),
        successors=("p:oomChild", "p:oooChild"),
    )
    oomChild = OneOrMore(
        "p:oomChild", successors=("p:oooChild", "p:zomChild", "p:zooChild")
    )
    oooChild = OneAndOnlyOne("p:oooChild")
    zomChild = ZeroOrMore("p:zomChild", successors=("p:zooChild",))
    zooChild = ZeroOrOne("p:zooChild", successors=())
    optAttr = OptionalAttribute("p:optAttr", ST_IntegerType)
    reqAttr = RequiredAttribute("reqAttr", ST_IntegerType)


class CT_Choice(BaseOxmlElement):
    """
    ``<p:choice>`` element
    """


class CT_OomChild(BaseOxmlElement):
    """
    Oom standing for 'OneOrMore', ``<p:oomChild>`` element, representing a
    child element that can appear multiple times in sequence, but must appear
    at least once.
    """


class CT_ZomChild(BaseOxmlElement):
    """
    Zom standing for 'ZeroOrMore', ``<p:zomChild>`` element, representing an
    optional child element that can appear multiple times in sequence.
    """


class CT_ZooChild(BaseOxmlElement):
    """
    Zoo standing for 'ZeroOrOne', ``<p:zooChild>`` element, an invented
    element for use in testing.
    """


register_element_cls("p:parent", CT_Parent)
register_element_cls("p:choice", CT_Choice)
register_element_cls("p:oomChild", CT_OomChild)
register_element_cls("p:zomChild", CT_ZomChild)
register_element_cls("p:zooChild", CT_ZooChild)


class CT_ChoiceBuilder(BaseBuilder):
    __tag__ = "p:choice"
    __nspfxs__ = ("p",)
    __attrs__ = ()


class CT_Choice2Builder(BaseBuilder):
    __tag__ = "p:choice2"
    __nspfxs__ = ("p",)
    __attrs__ = ()


class CT_ParentBuilder(BaseBuilder):
    __tag__ = "p:parent"
    __nspfxs__ = ("p",)
    __attrs__ = ("p:optAttr", "reqAttr")


class CT_OomChildBuilder(BaseBuilder):
    __tag__ = "p:oomChild"
    __nspfxs__ = ("p",)
    __attrs__ = ()


class CT_OooChildBuilder(BaseBuilder):
    __tag__ = "p:oooChild"
    __nspfxs__ = ("p",)
    __attrs__ = ()


class CT_ZomChildBuilder(BaseBuilder):
    __tag__ = "p:zomChild"
    __nspfxs__ = ("p",)
    __attrs__ = ()


class CT_ZooChildBuilder(BaseBuilder):
    __tag__ = "p:zooChild"
    __nspfxs__ = ("p",)
    __attrs__ = ()


def a_choice():
    return CT_ChoiceBuilder()


def a_choice2():
    return CT_Choice2Builder()


def a_parent():
    return CT_ParentBuilder()


def a_zomChild():
    return CT_ZomChildBuilder()


def a_zooChild():
    return CT_ZooChildBuilder()


def an_oomChild():
    return CT_OomChildBuilder()


def an_oooChild():
    return CT_OooChildBuilder()



# encoding: utf-8

"""
Presentation part, the main part in a .pptx package.
"""

from __future__ import absolute_import

from ..opc.constants import RELATIONSHIP_TYPE as RT
from ..opc.package import XmlPart
from ..opc.packuri import PackURI
from ..presentation import Presentation
from .slide import NotesMasterPart, SlidePart
from ..util import lazyproperty


class PresentationPart(XmlPart):
    """
    Top level class in object model, represents the contents of the /ppt
    directory of a .pptx file.
    """

    def add_slide(self, slide_layout):
        """
        Return an (rId, slide) pair of a newly created blank slide that
        inherits appearance from *slide_layout*.
        """
        partname = self._next_slide_partname
        slide_layout_part = slide_layout.part
        slide_part = SlidePart.new(partname, self.package, slide_layout_part)
        rId = self.relate_to(slide_part, RT.SLIDE)
        return rId, slide_part.slide

    @property
    def core_properties(self):
        """
        A |CoreProperties| object providing read/write access to the core
        properties of this presentation.
        """
        return self.package.core_properties

    def get_slide(self, slide_id):
        """
        Return the |Slide| object identified by *slide_id* (in this
        presentation), or |None| if not found.
        """
        for sldId in self._element.sldIdLst:
            if sldId.id == slide_id:
                return self.related_parts[sldId.rId].slide
        return None

    @lazyproperty
    def notes_master(self):
        """
        Return the |NotesMaster| object for this presentation. If the
        presentation does not have a notes master, one is created from
        a default template. The same single instance is returned on each
        call.
        """
        return self.notes_master_part.notes_master

    @lazyproperty
    def notes_master_part(self):
        """
        Return the |NotesMasterPart| object for this presentation. If the
        presentation does not have a notes master, one is created from
        a default template. The same single instance is returned on each
        call.
        """
        try:
            return self.part_related_by(RT.NOTES_MASTER)
        except KeyError:
            notes_master_part = NotesMasterPart.create_default(self.package)
            self.relate_to(notes_master_part, RT.NOTES_MASTER)
            return notes_master_part

    @lazyproperty
    def presentation(self):
        """
        A |Presentation| object providing access to the content of this
        presentation.
        """
        return Presentation(self._element, self)

    def related_slide(self, rId):
        """
        Return the |Slide| object for the related |SlidePart| corresponding
        to relationship key *rId*.
        """
        return self.related_parts[rId].slide

    def related_slide_master(self, rId):
        """
        Return the |SlideMaster| object for the related |SlideMasterPart|
        corresponding to relationship key *rId*.
        """
        return self.related_parts[rId].slide_master

    def rename_slide_parts(self, rIds):
        """
        Assign incrementing partnames like ``/ppt/slides/slide9.xml`` to the
        slide parts identified by *rIds*, in the order their id appears in
        that sequence. The name portion is always ``slide``. The number part
        forms a continuous sequence starting at 1 (e.g. 1, 2, ... 10, ...).
        The extension is always ``.xml``.
        """
        for idx, rId in enumerate(rIds):
            slide_part = self.related_parts[rId]
            slide_part.partname = PackURI("/ppt/slides/slide%d.xml" % (idx + 1))

    def save(self, path_or_stream):
        """
        Save this presentation package to *path_or_stream*, which can be
        either a path to a filesystem location (a string) or a file-like
        object.
        """
        self.package.save(path_or_stream)

    def slide_id(self, slide_part):
        """
        Return the slide identifier associated with *slide_part* in this
        presentation.
        """
        for sldId in self._element.sldIdLst:
            if self.related_parts[sldId.rId] is slide_part:
                return sldId.id
        raise ValueError("matching slide_part not found")

    @property
    def _next_slide_partname(self):
        """
        Return |PackURI| instance containing the partname for a slide to be
        appended to this slide collection, e.g. ``/ppt/slides/slide9.xml``
        for a slide collection containing 8 slides.
        """
        sldIdLst = self._element.get_or_add_sldIdLst()
        partname_str = "/ppt/slides/slide%d.xml" % (len(sldIdLst) + 1)
        return PackURI(partname_str)



# encoding: utf-8

"""Unit-test suite for `pptx.opc.package` module."""

import io

import pytest

from pptx.opc.oxml import CT_Relationships
from pptx.opc.packuri import PACKAGE_URI, PackURI
from pptx.opc.package import (
    OpcPackage,
    Part,
    PartFactory,
    _Relationship,
    RelationshipCollection,
    Unmarshaller,
    XmlPart,
)
from pptx.opc.pkgreader import PackageReader
from pptx.oxml.xmlchemy import BaseOxmlElement
from pptx.package import Package

from ..unitutil.cxml import element
from ..unitutil.file import absjoin, test_file_dir
from ..unitutil.mock import (
    call,
    class_mock,
    cls_attr_mock,
    function_mock,
    initializer_mock,
    instance_mock,
    loose_mock,
    method_mock,
    Mock,
    patch,
    PropertyMock,
)


class DescribeOpcPackage(object):
    def it_can_open_a_pkg_file(self, PackageReader_, PartFactory_, Unmarshaller_):
        # mockery ----------------------
        pkg_file = Mock(name="pkg_file")
        pkg_reader = PackageReader_.from_file.return_value
        # exercise ---------------------
        pkg = OpcPackage.open(pkg_file)
        # verify -----------------------
        PackageReader_.from_file.assert_called_once_with(pkg_file)
        Unmarshaller_.unmarshal.assert_called_once_with(pkg_reader, pkg, PartFactory_)
        assert isinstance(pkg, OpcPackage)

    def it_initializes_its_rels_collection_on_first_reference(
        self, RelationshipCollection_
    ):
        pkg = OpcPackage()
        rels = pkg.rels
        RelationshipCollection_.assert_called_once_with(PACKAGE_URI.baseURI)
        assert rels == RelationshipCollection_.return_value

    def it_can_add_a_relationship_to_a_part(self, pkg_with_rels_, rel_attrs_):
        reltype, target, rId = rel_attrs_
        pkg = pkg_with_rels_
        # exercise ---------------------
        pkg.load_rel(reltype, target, rId)
        # verify -----------------------
        pkg._rels.add_relationship.assert_called_once_with(reltype, target, rId, False)

    def it_can_establish_a_relationship_to_another_part(self, relate_to_part_fixture_):
        pkg, part_, reltype, rId = relate_to_part_fixture_
        _rId = pkg.relate_to(part_, reltype)
        pkg.rels.get_or_add.assert_called_once_with(reltype, part_)
        assert _rId == rId

    def it_can_provide_a_list_of_the_parts_it_contains(self):
        # mockery ----------------------
        parts = [Mock(name="part1"), Mock(name="part2")]
        pkg = OpcPackage()
        # verify -----------------------
        with patch.object(OpcPackage, "iter_parts", return_value=parts):
            assert pkg.parts == [parts[0], parts[1]]

    def it_can_iterate_over_its_parts(self, iter_parts_fixture):
        package, expected_parts = iter_parts_fixture
        parts = list(package.iter_parts())
        assert parts == expected_parts

    def it_can_iterate_over_its_relationships(self, iter_rels_fixture):
        package, expected_rels = iter_rels_fixture
        rels = list(package.iter_rels())
        assert rels == expected_rels

    def it_can_find_a_part_related_by_reltype(self, related_part_fixture_):
        pkg, reltype, related_part_ = related_part_fixture_
        related_part = pkg.part_related_by(reltype)
        pkg.rels.part_with_reltype.assert_called_once_with(reltype)
        assert related_part is related_part_

    def it_can_find_the_next_available_vector_partname(self, next_partname_fixture):
        package, partname_template, expected_partname = next_partname_fixture
        partname = package.next_partname(partname_template)
        assert isinstance(partname, PackURI)
        assert partname == expected_partname

    def it_can_save_to_a_pkg_file(self, pkg_file_, PackageWriter_, parts, parts_):
        pkg = OpcPackage()
        pkg.save(pkg_file_)
        for part in parts_:
            part.before_marshal.assert_called_once_with()
        PackageWriter_.write.assert_called_once_with(pkg_file_, pkg._rels, parts_)

    def it_can_be_notified_after_unmarshalling_is_complete(self, pkg):
        pkg.after_unmarshal()

    # fixtures ---------------------------------------------

    @pytest.fixture
    def iter_parts_fixture(self, request, rels_fixture):
        package, parts, rels = rels_fixture
        expected_parts = list(parts)
        return package, expected_parts

    @pytest.fixture
    def iter_rels_fixture(self, request, rels_fixture):
        package, parts, rels = rels_fixture
        expected_rels = list(rels)
        return package, expected_rels

    @pytest.fixture(params=[((), 1), ((1,), 2), ((1, 2), 3), ((2, 3), 1), ((1, 3), 2)])
    def next_partname_fixture(self, request, iter_parts_):
        existing_partname_numbers, next_partname_number = request.param
        package = OpcPackage()
        parts = [
            instance_mock(
                request, Part, name="part[%d]" % idx, partname="/foo/bar/baz%d.xml" % n
            )
            for idx, n in enumerate(existing_partname_numbers)
        ]
        iter_parts_.return_value = iter(parts)
        partname_template = "/foo/bar/baz%d.xml"
        expected_partname = PackURI("/foo/bar/baz%d.xml" % next_partname_number)
        return package, partname_template, expected_partname

    @pytest.fixture
    def relate_to_part_fixture_(self, request, pkg, rels_, reltype):
        rId = "rId99"
        rel_ = instance_mock(request, _Relationship, name="rel_", rId=rId)
        rels_.get_or_add.return_value = rel_
        pkg._rels = rels_
        part_ = instance_mock(request, Part, name="part_")
        return pkg, part_, reltype, rId

    @pytest.fixture
    def related_part_fixture_(self, request, rels_, reltype):
        related_part_ = instance_mock(request, Part, name="related_part_")
        rels_.part_with_reltype.return_value = related_part_
        pkg = OpcPackage()
        pkg._rels = rels_
        return pkg, reltype, related_part_

    @pytest.fixture
    def rels_fixture(self, request, part_1_, part_2_):
        """
        +----------+          +--------+
        | pkg_rels |-- r1 --> | part_1 |
        +----------+          +--------+
             |     |            |    ^
          r5 |     | r4      r2 |    | r3
             |     |            |    |
             v     |            v    |
         external  |          +--------+
                   +--------> | part_2 |
                              +--------+
        """
        r1 = self.rel(request, False, part_1_, "r1")
        r2 = self.rel(request, False, part_2_, "r2")
        r3 = self.rel(request, False, part_1_, "r3")
        r4 = self.rel(request, False, part_2_, "r4")
        r5 = self.rel(request, True, None, "r5")

        package = OpcPackage()

        package._rels = self.rels(request, (r1, r4, r5))
        part_1_.rels = self.rels(request, (r2,))
        part_2_.rels = self.rels(request, (r3,))

        return package, (part_1_, part_2_), (r1, r2, r3, r4, r5)

    # fixture components -----------------------------------

    @pytest.fixture
    def iter_parts_(self, request):
        return method_mock(request, OpcPackage, "iter_parts")

    @pytest.fixture
    def PackageReader_(self, request):
        return class_mock(request, "pptx.opc.package.PackageReader")

    @pytest.fixture
    def PackageWriter_(self, request):
        return class_mock(request, "pptx.opc.package.PackageWriter")

    @pytest.fixture
    def PartFactory_(self, request):
        return class_mock(request, "pptx.opc.package.PartFactory")

    @pytest.fixture
    def part_1_(self, request):
        return instance_mock(request, Part)

    @pytest.fixture
    def part_2_(self, request):
        return instance_mock(request, Part)

    @pytest.fixture
    def parts(self, request, parts_):
        """
        Return a mock patching property OpcPackage.parts, reversing the
        patch after each use.
        """
        _patch = patch.object(
            OpcPackage, "parts", new_callable=PropertyMock, return_value=parts_
        )
        request.addfinalizer(_patch.stop)
        return _patch.start()

    @pytest.fixture
    def parts_(self, request):
        part_ = instance_mock(request, Part, name="part_")
        part_2_ = instance_mock(request, Part, name="part_2_")
        return [part_, part_2_]

    @pytest.fixture
    def pkg(self, request):
        return OpcPackage()

    @pytest.fixture
    def pkg_file_(self, request):
        return loose_mock(request)

    @pytest.fixture
    def pkg_with_rels_(self, request, rels_):
        pkg = OpcPackage()
        pkg._rels = rels_
        return pkg

    @pytest.fixture
    def RelationshipCollection_(self, request):
        return class_mock(request, "pptx.opc.package.RelationshipCollection")

    @pytest.fixture
    def rel_attrs_(self, request):
        reltype = "http://rel/type"
        target_ = instance_mock(request, Part, name="target_")
        rId = "rId99"
        return reltype, target_, rId

    def rel(self, request, is_external, target_part, name):
        return instance_mock(
            request,
            _Relationship,
            is_external=is_external,
            target_part=target_part,
            name=name,
        )

    def rels(self, request, values):
        rels = instance_mock(request, RelationshipCollection)
        rels.values.return_value = values
        return rels

    @pytest.fixture
    def rels_(self, request):
        return instance_mock(request, RelationshipCollection)

    @pytest.fixture
    def reltype(self, request):
        return "http://rel/type"

    @pytest.fixture
    def Unmarshaller_(self, request):
        return class_mock(request, "pptx.opc.package.Unmarshaller")


class DescribePart(object):
    """Unit-test suite for `pptx.opc.package.Part` objects."""

    def it_can_be_constructed_by_PartFactory(self, load_fixture):
        partname_, content_type_, blob_, package_, __init_ = load_fixture
        part = Part.load(partname_, content_type_, blob_, package_)
        __init_.assert_called_once_with(partname_, content_type_, blob_, package_)
        assert isinstance(part, Part)

    def it_knows_its_partname(self, partname_get_fixture):
        part, expected_partname = partname_get_fixture
        assert part.partname == expected_partname

    def it_can_change_its_partname(self, partname_set_fixture):
        part, new_partname = partname_set_fixture
        part.partname = new_partname
        assert part.partname == new_partname

    def it_knows_its_content_type(self, content_type_fixture):
        part, expected_content_type = content_type_fixture
        assert part.content_type == expected_content_type

    def it_knows_the_package_it_belongs_to(self, package_get_fixture):
        part, expected_package = package_get_fixture
        assert part.package == expected_package

    def it_can_be_notified_after_unmarshalling_is_complete(self, part):
        part.after_unmarshal()

    def it_can_be_notified_before_marshalling_is_started(self, part):
        part.before_marshal()

    def it_uses_the_load_blob_as_its_blob(self, blob_fixture):
        part, load_blob = blob_fixture
        assert part.blob is load_blob

    def it_can_change_its_blob(self):
        part, new_blob = Part(None, None, "xyz", None), "foobar"
        part.blob = new_blob
        assert part.blob == new_blob

    def it_can_load_a_blob_from_a_file_path_to_help(self):
        path = absjoin(test_file_dir, "minimal.pptx")
        with open(path, "rb") as f:
            file_bytes = f.read()
        part = Part(None, None, None, None)

        assert part._blob_from_file(path) == file_bytes

    def it_can_load_a_blob_from_a_file_like_object_to_help(self):
        part = Part(None, None, None, None)
        assert part._blob_from_file(io.BytesIO(b"012345")) == b"012345"

    # fixtures ---------------------------------------------

    @pytest.fixture
    def blob_fixture(self, blob_):
        part = Part(None, None, blob_, None)
        return part, blob_

    @pytest.fixture
    def content_type_fixture(self):
        content_type = "content/type"
        part = Part(None, content_type, None, None)
        return part, content_type

    @pytest.fixture
    def load_fixture(self, request, partname_, content_type_, blob_, package_, __init_):
        return (partname_, content_type_, blob_, package_, __init_)

    @pytest.fixture
    def package_get_fixture(self, package_):
        part = Part(None, None, None, package_)
        return part, package_

    @pytest.fixture
    def part(self):
        part = Part(None, None)
        return part

    @pytest.fixture
    def partname_get_fixture(self):
        partname = PackURI("/part/name")
        part = Part(partname, None, None, None)
        return part, partname

    @pytest.fixture
    def partname_set_fixture(self):
        old_partname = PackURI("/old/part/name")
        new_partname = PackURI("/new/part/name")
        part = Part(old_partname, None, None, None)
        return part, new_partname

    # fixture components ---------------------------------------------

    @pytest.fixture
    def blob_(self, request):
        return instance_mock(request, bytes)

    @pytest.fixture
    def content_type_(self, request):
        return instance_mock(request, str)

    @pytest.fixture
    def __init_(self, request):
        return initializer_mock(request, Part)

    @pytest.fixture
    def package_(self, request):
        return instance_mock(request, OpcPackage)

    @pytest.fixture
    def partname_(self, request):
        return instance_mock(request, PackURI)


class DescribePartRelationshipManagementInterface(object):
    def it_provides_access_to_its_relationships(self, rels_fixture):
        part, Relationships_, partname_, rels_ = rels_fixture
        rels = part.rels
        Relationships_.assert_called_once_with(partname_.baseURI)
        assert rels is rels_

    def it_can_load_a_relationship(self, load_rel_fixture):
        part, rels_, reltype_, target_, rId_ = load_rel_fixture
        part.load_rel(reltype_, target_, rId_)
        rels_.add_relationship.assert_called_once_with(reltype_, target_, rId_, False)

    def it_can_establish_a_relationship_to_another_part(self, relate_to_part_fixture):
        part, target_, reltype_, rId_ = relate_to_part_fixture
        rId = part.relate_to(target_, reltype_)
        part.rels.get_or_add.assert_called_once_with(reltype_, target_)
        assert rId is rId_

    def it_can_establish_an_external_relationship(self, relate_to_url_fixture):
        part, url_, reltype_, rId_ = relate_to_url_fixture
        rId = part.relate_to(url_, reltype_, is_external=True)
        part.rels.get_or_add_ext_rel.assert_called_once_with(reltype_, url_)
        assert rId is rId_

    def it_can_drop_a_relationship(self, drop_rel_fixture):
        part, rId, rel_should_be_gone = drop_rel_fixture
        part.drop_rel(rId)
        if rel_should_be_gone:
            assert rId not in part.rels
        else:
            assert rId in part.rels

    def it_can_find_a_related_part_by_reltype(self, related_part_fixture):
        part, reltype_, related_part_ = related_part_fixture
        related_part = part.part_related_by(reltype_)
        part.rels.part_with_reltype.assert_called_once_with(reltype_)
        assert related_part is related_part_

    def it_can_find_a_related_part_by_rId(self, related_parts_fixture):
        part, related_parts_ = related_parts_fixture
        assert part.related_parts is related_parts_

    def it_can_find_the_uri_of_an_external_relationship(self, target_ref_fixture):
        part, rId_, url_ = target_ref_fixture
        url = part.target_ref(rId_)
        assert url == url_

    # fixtures ---------------------------------------------

    @pytest.fixture(
        params=[
            ("p:sp", True),
            ("p:sp/r:a{r:id=rId42}", True),
            ("p:sp/r:a{r:id=rId42}/r:b{r:id=rId42}", False),
        ]
    )
    def drop_rel_fixture(self, request, part):
        part_cxml, rel_should_be_dropped = request.param
        rId = "rId42"
        part._element = element(part_cxml)
        part._rels = {rId: None}
        return part, rId, rel_should_be_dropped

    @pytest.fixture
    def load_rel_fixture(self, part, rels_, reltype_, part_, rId_):
        part._rels = rels_
        return part, rels_, reltype_, part_, rId_

    @pytest.fixture
    def relate_to_part_fixture(self, request, part, reltype_, part_, rels_, rId_):
        part._rels = rels_
        target_ = part_
        return part, target_, reltype_, rId_

    @pytest.fixture
    def relate_to_url_fixture(self, request, part, rels_, url_, reltype_, rId_):
        part._rels = rels_
        return part, url_, reltype_, rId_

    @pytest.fixture
    def related_part_fixture(self, request, part, rels_, reltype_, part_):
        part._rels = rels_
        return part, reltype_, part_

    @pytest.fixture
    def related_parts_fixture(self, request, part, rels_, related_parts_):
        part._rels = rels_
        return part, related_parts_

    @pytest.fixture
    def rels_fixture(self, Relationships_, partname_, rels_):
        part = Part(partname_, None)
        return part, Relationships_, partname_, rels_

    @pytest.fixture
    def target_ref_fixture(self, request, part, rId_, rel_, url_):
        part._rels = {rId_: rel_}
        return part, rId_, url_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def part(self):
        return Part(None, None)

    @pytest.fixture
    def part_(self, request):
        return instance_mock(request, Part)

    @pytest.fixture
    def partname_(self, request):
        return instance_mock(request, PackURI)

    @pytest.fixture
    def Relationships_(self, request, rels_):
        return class_mock(
            request, "pptx.opc.package.RelationshipCollection", return_value=rels_
        )

    @pytest.fixture
    def rel_(self, request, rId_, url_):
        return instance_mock(request, _Relationship, rId=rId_, target_ref=url_)

    @pytest.fixture
    def rels_(self, request, part_, rel_, rId_, related_parts_):
        rels_ = instance_mock(request, RelationshipCollection)
        rels_.part_with_reltype.return_value = part_
        rels_.get_or_add.return_value = rel_
        rels_.get_or_add_ext_rel.return_value = rId_
        rels_.related_parts = related_parts_
        return rels_

    @pytest.fixture
    def related_parts_(self, request):
        return instance_mock(request, dict)

    @pytest.fixture
    def reltype_(self, request):
        return instance_mock(request, str)

    @pytest.fixture
    def rId_(self, request):
        return instance_mock(request, str)

    @pytest.fixture
    def url_(self, request):
        return instance_mock(request, str)


class DescribeXmlPart(object):
    def it_can_be_constructed_by_PartFactory(self, load_fixture):
        partname_, content_type_, blob_, package_ = load_fixture[:4]
        element_, parse_xml_, __init_ = load_fixture[4:]
        # exercise ---------------------
        part = XmlPart.load(partname_, content_type_, blob_, package_)
        # verify -----------------------
        parse_xml_.assert_called_once_with(blob_)
        __init_.assert_called_once_with(partname_, content_type_, element_, package_)
        assert isinstance(part, XmlPart)

    def it_can_serialize_to_xml(self, blob_fixture):
        xml_part, element_, serialize_part_xml_ = blob_fixture
        blob = xml_part.blob
        serialize_part_xml_.assert_called_once_with(element_)
        assert blob is serialize_part_xml_.return_value

    def it_knows_its_the_part_for_its_child_objects(self, part_fixture):
        xml_part = part_fixture
        assert xml_part.part is xml_part

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def blob_fixture(self, request, element_, serialize_part_xml_):
        xml_part = XmlPart(None, None, element_, None)
        return xml_part, element_, serialize_part_xml_

    @pytest.fixture
    def load_fixture(
        self,
        request,
        partname_,
        content_type_,
        blob_,
        package_,
        element_,
        parse_xml_,
        __init_,
    ):
        return (
            partname_,
            content_type_,
            blob_,
            package_,
            element_,
            parse_xml_,
            __init_,
        )

    @pytest.fixture
    def part_fixture(self):
        return XmlPart(None, None, None, None)

    # fixture components ---------------------------------------------

    @pytest.fixture
    def blob_(self, request):
        return instance_mock(request, str)

    @pytest.fixture
    def content_type_(self, request):
        return instance_mock(request, str)

    @pytest.fixture
    def element_(self, request):
        return instance_mock(request, BaseOxmlElement)

    @pytest.fixture
    def __init_(self, request):
        return initializer_mock(request, XmlPart)

    @pytest.fixture
    def package_(self, request):
        return instance_mock(request, OpcPackage)

    @pytest.fixture
    def parse_xml_(self, request, element_):
        return function_mock(
            request, "pptx.opc.package.parse_xml", return_value=element_
        )

    @pytest.fixture
    def partname_(self, request):
        return instance_mock(request, PackURI)

    @pytest.fixture
    def serialize_part_xml_(self, request):
        return function_mock(request, "pptx.opc.package.serialize_part_xml")


class DescribePartFactory(object):
    def it_constructs_custom_part_type_for_registered_content_types(
        self, part_args_, CustomPartClass_, part_of_custom_type_
    ):
        # fixture ----------------------
        partname, content_type, pkg, blob = part_args_
        # exercise ---------------------
        PartFactory.part_type_for[content_type] = CustomPartClass_
        part = PartFactory(partname, content_type, pkg, blob)
        # verify -----------------------
        CustomPartClass_.load.assert_called_once_with(partname, content_type, pkg, blob)
        assert part is part_of_custom_type_

    def it_constructs_part_using_default_class_when_no_custom_registered(
        self, part_args_2_, DefaultPartClass_, part_of_default_type_
    ):
        partname, content_type, pkg, blob = part_args_2_
        part = PartFactory(partname, content_type, pkg, blob)
        DefaultPartClass_.load.assert_called_once_with(
            partname, content_type, pkg, blob
        )
        assert part is part_of_default_type_

    # fixtures ---------------------------------------------

    @pytest.fixture
    def part_of_custom_type_(self, request):
        return instance_mock(request, Part)

    @pytest.fixture
    def CustomPartClass_(self, request, part_of_custom_type_):
        CustomPartClass_ = Mock(name="CustomPartClass", spec=Part)
        CustomPartClass_.load.return_value = part_of_custom_type_
        return CustomPartClass_

    @pytest.fixture
    def part_of_default_type_(self, request):
        return instance_mock(request, Part)

    @pytest.fixture
    def DefaultPartClass_(self, request, part_of_default_type_):
        DefaultPartClass_ = cls_attr_mock(request, PartFactory, "default_part_type")
        DefaultPartClass_.load.return_value = part_of_default_type_
        return DefaultPartClass_

    @pytest.fixture
    def part_args_(self, request):
        partname_ = PackURI("/foo/bar.xml")
        content_type_ = "content/type"
        pkg_ = instance_mock(request, Package, name="pkg_")
        blob_ = b"blob_"
        return partname_, content_type_, pkg_, blob_

    @pytest.fixture
    def part_args_2_(self, request):
        partname_2_ = PackURI("/bar/foo.xml")
        content_type_2_ = "foobar/type"
        pkg_2_ = instance_mock(request, Package, name="pkg_2_")
        blob_2_ = b"blob_2_"
        return partname_2_, content_type_2_, pkg_2_, blob_2_


class Describe_Relationship(object):
    def it_remembers_construction_values(self):
        # test data --------------------
        rId = "rId9"
        reltype = "reltype"
        target = Mock(name="target_part")
        external = False
        # exercise ---------------------
        rel = _Relationship(rId, reltype, target, None, external)
        # verify -----------------------
        assert rel.rId == rId
        assert rel.reltype == reltype
        assert rel.target_part == target
        assert rel.is_external == external

    def it_should_raise_on_target_part_access_on_external_rel(self):
        rel = _Relationship(None, None, None, None, external=True)
        with pytest.raises(ValueError):
            rel.target_part

    def it_should_have_target_ref_for_external_rel(self):
        rel = _Relationship(None, None, "target", None, external=True)
        assert rel.target_ref == "target"

    def it_should_have_relative_ref_for_internal_rel(self):
        """
        Internal relationships (TargetMode == 'Internal' in the XML) should
        have a relative ref, e.g. '../slideLayouts/slideLayout1.xml', for
        the target_ref attribute.
        """
        part = Mock(name="part", partname=PackURI("/ppt/media/image1.png"))
        baseURI = "/ppt/slides"
        rel = _Relationship(None, None, part, baseURI)  # external=False
        assert rel.target_ref == "../media/image1.png"


class DescribeRelationshipCollection(object):
    def it_has_a_len(self):
        rels = RelationshipCollection(None)
        assert len(rels) == 0

    def it_has_dict_style_lookup_of_rel_by_rId(self):
        rel = Mock(name="rel", rId="foobar")
        rels = RelationshipCollection(None)
        rels["foobar"] = rel
        assert rels["foobar"] == rel

    def it_should_raise_on_failed_lookup_by_rId(self):
        rels = RelationshipCollection(None)
        with pytest.raises(KeyError):
            rels["barfoo"]

    def it_can_add_a_relationship(self, _Relationship_):
        baseURI, rId, reltype, target, external = (
            "baseURI",
            "rId9",
            "reltype",
            "target",
            False,
        )
        rels = RelationshipCollection(baseURI)
        rel = rels.add_relationship(reltype, target, rId, external)
        _Relationship_.assert_called_once_with(rId, reltype, target, baseURI, external)
        assert rels[rId] == rel
        assert rel == _Relationship_.return_value

    def it_can_add_an_external_relationship(self, add_ext_rel_fixture_):
        rels, reltype, url = add_ext_rel_fixture_
        rId = rels.get_or_add_ext_rel(reltype, url)
        rel = rels[rId]
        assert rel.is_external
        assert rel.target_ref == url
        assert rel.reltype == reltype

    def it_should_return_an_existing_one_if_it_matches(
        self, add_matching_ext_rel_fixture_
    ):
        rels, reltype, url, rId = add_matching_ext_rel_fixture_
        _rId = rels.get_or_add_ext_rel(reltype, url)
        assert _rId == rId
        assert len(rels) == 1

    def it_can_compose_rels_xml(self, rels, rels_elm):
        # exercise ---------------------
        rels.xml
        # verify -----------------------
        rels_elm.assert_has_calls(
            [
                call.add_rel("rId1", "http://rt-hyperlink", "http://some/link", True),
                call.add_rel("rId2", "http://rt-image", "../media/image1.png", False),
                call.xml(),
            ],
            any_order=True,
        )

    # fixtures ---------------------------------------------

    @pytest.fixture
    def add_ext_rel_fixture_(self, reltype, url):
        rels = RelationshipCollection(None)
        return rels, reltype, url

    @pytest.fixture
    def add_matching_ext_rel_fixture_(self, request, reltype, url):
        rId = "rId369"
        rels = RelationshipCollection(None)
        rels.add_relationship(reltype, url, rId, is_external=True)
        return rels, reltype, url, rId

    @pytest.fixture
    def _Relationship_(self, request):
        return class_mock(request, "pptx.opc.package._Relationship")

    @pytest.fixture
    def rels(self):
        """
        Populated RelationshipCollection instance that will exercise the
        rels.xml property.
        """
        rels = RelationshipCollection("/baseURI")
        rels.add_relationship(
            reltype="http://rt-hyperlink",
            target="http://some/link",
            rId="rId1",
            is_external=True,
        )
        part = Mock(name="part")
        part.partname.relative_ref.return_value = "../media/image1.png"
        rels.add_relationship(reltype="http://rt-image", target=part, rId="rId2")
        return rels

    @pytest.fixture
    def rels_elm(self, request):
        """
        Return a rels_elm mock that will be returned from
        CT_Relationships.new()
        """
        # create rels_elm mock with a .xml property
        rels_elm = Mock(name="rels_elm")
        xml = PropertyMock(name="xml")
        type(rels_elm).xml = xml
        rels_elm.attach_mock(xml, "xml")
        rels_elm.reset_mock()  # to clear attach_mock call
        # patch CT_Relationships to return that rels_elm
        patch_ = patch.object(CT_Relationships, "new", return_value=rels_elm)
        patch_.start()
        request.addfinalizer(patch_.stop)
        return rels_elm

    @pytest.fixture
    def reltype(self):
        return "http://rel/type"

    @pytest.fixture
    def url(self):
        return "https://github.com/scanny/python-pptx"


class DescribeUnmarshaller(object):
    def it_can_unmarshal_from_a_pkg_reader(
        self,
        pkg_reader_,
        pkg_,
        part_factory_,
        _unmarshal_parts,
        _unmarshal_relationships,
        parts_dict_,
    ):
        # exercise ---------------------
        Unmarshaller.unmarshal(pkg_reader_, pkg_, part_factory_)
        # verify -----------------------
        _unmarshal_parts.assert_called_once_with(pkg_reader_, pkg_, part_factory_)
        _unmarshal_relationships.assert_called_once_with(pkg_reader_, pkg_, parts_dict_)
        for part in parts_dict_.values():
            part.after_unmarshal.assert_called_once_with()
        pkg_.after_unmarshal.assert_called_once_with()

    def it_can_unmarshal_parts(
        self,
        pkg_reader_,
        pkg_,
        part_factory_,
        parts_dict_,
        partnames_,
        content_types_,
        blobs_,
    ):
        # fixture ----------------------
        partname_, partname_2_ = partnames_
        content_type_, content_type_2_ = content_types_
        blob_, blob_2_ = blobs_
        # exercise ---------------------
        parts = Unmarshaller._unmarshal_parts(pkg_reader_, pkg_, part_factory_)
        # verify -----------------------
        assert part_factory_.call_args_list == [
            call(partname_, content_type_, blob_, pkg_),
            call(partname_2_, content_type_2_, blob_2_, pkg_),
        ]
        assert parts == parts_dict_

    def it_can_unmarshal_relationships(self):
        # test data --------------------
        reltype = "http://reltype"
        # mockery ----------------------
        pkg_reader = Mock(name="pkg_reader")
        pkg_reader.iter_srels.return_value = (
            (
                "/",
                Mock(
                    name="srel1",
                    rId="rId1",
                    reltype=reltype,
                    target_partname="partname1",
                    is_external=False,
                ),
            ),
            (
                "/",
                Mock(
                    name="srel2",
                    rId="rId2",
                    reltype=reltype,
                    target_ref="target_ref_1",
                    is_external=True,
                ),
            ),
            (
                "partname1",
                Mock(
                    name="srel3",
                    rId="rId3",
                    reltype=reltype,
                    target_partname="partname2",
                    is_external=False,
                ),
            ),
            (
                "partname2",
                Mock(
                    name="srel4",
                    rId="rId4",
                    reltype=reltype,
                    target_ref="target_ref_2",
                    is_external=True,
                ),
            ),
        )
        pkg = Mock(name="pkg")
        parts = {}
        for num in range(1, 3):
            name = "part%d" % num
            part = Mock(name=name)
            parts["partname%d" % num] = part
            pkg.attach_mock(part, name)
        # exercise ---------------------
        Unmarshaller._unmarshal_relationships(pkg_reader, pkg, parts)
        # verify -----------------------
        expected_pkg_calls = [
            call.load_rel(reltype, parts["partname1"], "rId1", False),
            call.load_rel(reltype, "target_ref_1", "rId2", True),
            call.part1.load_rel(reltype, parts["partname2"], "rId3", False),
            call.part2.load_rel(reltype, "target_ref_2", "rId4", True),
        ]
        assert pkg.mock_calls == expected_pkg_calls

    # fixtures ---------------------------------------------

    @pytest.fixture
    def blobs_(self, request):
        blob_ = loose_mock(request, spec=str, name="blob_")
        blob_2_ = loose_mock(request, spec=str, name="blob_2_")
        return blob_, blob_2_

    @pytest.fixture
    def content_types_(self, request):
        content_type_ = loose_mock(request, spec=str, name="content_type_")
        content_type_2_ = loose_mock(request, spec=str, name="content_type_2_")
        return content_type_, content_type_2_

    @pytest.fixture
    def part_factory_(self, request, parts_):
        part_factory_ = loose_mock(request, spec=Part)
        part_factory_.side_effect = parts_
        return part_factory_

    @pytest.fixture
    def partnames_(self, request):
        partname_ = loose_mock(request, spec=str, name="partname_")
        partname_2_ = loose_mock(request, spec=str, name="partname_2_")
        return partname_, partname_2_

    @pytest.fixture
    def parts_(self, request):
        part_ = instance_mock(request, Part, name="part_")
        part_2_ = instance_mock(request, Part, name="part_2")
        return part_, part_2_

    @pytest.fixture
    def parts_dict_(self, request, partnames_, parts_):
        partname_, partname_2_ = partnames_
        part_, part_2_ = parts_
        return {partname_: part_, partname_2_: part_2_}

    @pytest.fixture
    def pkg_(self, request):
        return instance_mock(request, Package)

    @pytest.fixture
    def pkg_reader_(self, request, partnames_, content_types_, blobs_):
        partname_, partname_2_ = partnames_
        content_type_, content_type_2_ = content_types_
        blob_, blob_2_ = blobs_
        spart_return_values = (
            (partname_, content_type_, blob_),
            (partname_2_, content_type_2_, blob_2_),
        )
        pkg_reader_ = instance_mock(request, PackageReader)
        pkg_reader_.iter_sparts.return_value = spart_return_values
        return pkg_reader_

    @pytest.fixture
    def _unmarshal_parts(self, request, parts_dict_):
        return method_mock(
            request, Unmarshaller, "_unmarshal_parts", return_value=parts_dict_
        )

    @pytest.fixture
    def _unmarshal_relationships(self, request):
        return method_mock(request, Unmarshaller, "_unmarshal_relationships")



# encoding: utf-8

"""
Test suite for pptx.oxml.theme module
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.oxml.theme import CT_OfficeStyleSheet

from ..unitutil.file import snippet_text


class DescribeCT_OfficeStyleSheet(object):
    def it_can_create_a_default_theme_element(self, new_fixture):
        expected_xml = new_fixture
        theme = CT_OfficeStyleSheet.new_default()
        assert theme.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def new_fixture(self):
        expected_xml = snippet_text("default-theme")
        return expected_xml



# encoding: utf-8

"""
Helper objects for unit testing.
"""


def count(start=0, step=1):
    """
    Local implementation of `itertools.count()` to allow v2.6 compatibility.
    """
    n = start
    while True:
        yield n
        n += step



# encoding: utf-8

"""Test suite for pptx.oxml.shapes.shapetree module."""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.oxml.shapes.autoshape import CT_Shape
from pptx.oxml.shapes.graphfrm import CT_GraphicalObjectFrame
from pptx.oxml.shapes.groupshape import CT_GroupShape
from pptx.oxml.shapes.picture import CT_Picture

from ...unitutil.cxml import element, xml
from ...unitutil.mock import call, class_mock, instance_mock, method_mock, property_mock


class DescribeCT_GroupShape(object):
    def it_can_add_a_graphicFrame_element_containing_a_table(self, add_table_fixt):
        spTree, id_, name, rows, cols, x, y, cx, cy = add_table_fixt[:9]
        new_table_graphicFrame_ = add_table_fixt[9]
        insert_element_before_, graphicFrame_ = add_table_fixt[10:]

        graphicFrame = spTree.add_table(id_, name, rows, cols, x, y, cx, cy)

        new_table_graphicFrame_.assert_called_once_with(
            id_, name, rows, cols, x, y, cx, cy
        )
        insert_element_before_.assert_called_once_with(graphicFrame_, "p:extLst")
        assert graphicFrame is graphicFrame_

    def it_can_add_a_grpSp_element(self, add_grpSp_fixture):
        spTree, expected_grpSp_xml, expected_xml = add_grpSp_fixture

        grpSp = spTree.add_grpSp()

        assert grpSp.xml == expected_grpSp_xml
        assert spTree.xml == expected_xml

    def it_can_add_a_pic_element_representing_a_picture(self, add_pic_fixt):
        spTree, id_, name, desc, rId, x, y, cx, cy = add_pic_fixt[:9]
        CT_Picture_, insert_element_before_, pic_ = add_pic_fixt[9:]
        pic = spTree.add_pic(id_, name, desc, rId, x, y, cx, cy)
        CT_Picture_.new_pic.assert_called_once_with(id_, name, desc, rId, x, y, cx, cy)
        insert_element_before_.assert_called_once_with(pic_, "p:extLst")
        assert pic is pic_

    def it_can_add_an_sp_element_for_a_placeholder(self, add_placeholder_fixt):
        spTree, id_, name, ph_type, orient, sz, idx = add_placeholder_fixt[:7]
        CT_Shape_, insert_element_before_, sp_ = add_placeholder_fixt[7:]
        sp = spTree.add_placeholder(id_, name, ph_type, orient, sz, idx)
        CT_Shape_.new_placeholder_sp.assert_called_once_with(
            id_, name, ph_type, orient, sz, idx
        )
        insert_element_before_.assert_called_once_with(sp_, "p:extLst")
        assert sp is sp_

    def it_can_add_an_sp_element_for_an_autoshape(self, add_autoshape_fixt):
        spTree, id_, name, prst, x, y, cx, cy = add_autoshape_fixt[:8]
        CT_Shape_, insert_element_before_, sp_ = add_autoshape_fixt[8:]
        sp = spTree.add_autoshape(id_, name, prst, x, y, cx, cy)
        CT_Shape_.new_autoshape_sp.assert_called_once_with(
            id_, name, prst, x, y, cx, cy
        )
        insert_element_before_.assert_called_once_with(sp_, "p:extLst")
        assert sp is sp_

    def it_can_add_a_textbox_sp_element(self, add_textbox_fixt):
        spTree, id_, name, x, y, cx, cy, CT_Shape_ = add_textbox_fixt[:8]
        insert_element_before_, sp_ = add_textbox_fixt[8:]
        sp = spTree.add_textbox(id_, name, x, y, cx, cy)
        CT_Shape_.new_textbox_sp.assert_called_once_with(id_, name, x, y, cx, cy)
        insert_element_before_.assert_called_once_with(sp_, "p:extLst")
        assert sp is sp_

    def it_can_recalculate_its_pos_and_size(self, recalc_fixture):
        xSp, expected_xml, parent_sp, calls = recalc_fixture

        xSp.recalculate_extents()

        assert xSp.xml == expected_xml
        assert parent_sp.recalculate_extents.call_args_list == calls

    def it_calculates_its_child_extents_to_help(self, child_exts_fixture):
        xSp, expected_values = child_exts_fixture
        x, y, cx, cy = xSp._child_extents
        assert (x, y, cx, cy) == expected_values

    # fixtures ---------------------------------------------

    @pytest.fixture
    def add_autoshape_fixt(self, spTree, CT_Shape_, insert_element_before_, sp_):
        id_, name, prst = 42, "name", "prst"
        x, y, cx, cy = 9, 8, 7, 6
        return (
            spTree,
            id_,
            name,
            prst,
            x,
            y,
            cx,
            cy,
            CT_Shape_,
            insert_element_before_,
            sp_,
        )

    @pytest.fixture
    def add_grpSp_fixture(self):
        spTree = element("p:spTree{a:b=c,r:s=t}")
        expected_grpSp_xml = (
            # ---can't get the namespaces right with cxml, using full text--
            '<p:grpSp xmlns:p="http://schemas.openxmlformats.org/presentatio'
            'nml/2006/main" xmlns:a="http://schemas.openxmlformats.org/drawi'
            'ngml/2006/main" xmlns:r="http://schemas.openxmlformats.org/offi'
            'ceDocument/2006/relationships">\n  <p:nvGrpSpPr>\n    <p:cNvPr '
            'id="1" name="Group 0"/>\n    <p:cNvGrpSpPr/>\n    <p:nvPr/>\n  '
            '</p:nvGrpSpPr>\n  <p:grpSpPr>\n    <a:xfrm>\n      <a:off x="0"'
            ' y="0"/>\n      <a:ext cx="0" cy="0"/>\n      <a:chOff x="0" y='
            '"0"/>\n      <a:chExt cx="0" cy="0"/>\n    </a:xfrm>\n  </p:grp'
            "SpPr>\n</p:grpSp>\n"
        )
        expected_xml = xml(
            "p:spTree{a:b=c,r:s=t}/p:grpSp/(p:nvGrpSpPr/(p:cNvPr{id=1,name=G"
            "roup 0},p:cNvGrpSpPr,p:nvPr),p:grpSpPr/a:xfrm/(a:off{x=0,y=0},a"
            ":ext{cx=0,cy=0},a:chOff{x=0,y=0},a:chExt{cx=0,cy=0}))"
        )
        return spTree, expected_grpSp_xml, expected_xml

    @pytest.fixture
    def add_pic_fixt(self, spTree, CT_Picture_, insert_element_before_, pic_):
        id_, name, desc, rId = 42, "name", "desc", "rId6"
        x, y, cx, cy = 6, 7, 8, 9
        return (
            spTree,
            id_,
            name,
            desc,
            rId,
            x,
            y,
            cx,
            cy,
            CT_Picture_,
            insert_element_before_,
            pic_,
        )

    @pytest.fixture
    def add_placeholder_fixt(self, spTree, CT_Shape_, insert_element_before_, sp_):
        id_, name, ph_type = 42, "name", "type"
        orient, sz, idx = "orient", "sz", 24
        return (
            spTree,
            id_,
            name,
            ph_type,
            orient,
            sz,
            idx,
            CT_Shape_,
            insert_element_before_,
            sp_,
        )

    @pytest.fixture
    def add_table_fixt(
        self, spTree, CT_GraphicalObjectFrame_, insert_element_before_, graphicFrame_
    ):
        id_, name, rows, cols = 42, "name", 12, 23
        x, y, cx, cy = 5, 4, 3, 2
        new_table_graphicFrame_ = CT_GraphicalObjectFrame_.new_table_graphicFrame
        return (
            spTree,
            id_,
            name,
            rows,
            cols,
            x,
            y,
            cx,
            cy,
            new_table_graphicFrame_,
            insert_element_before_,
            graphicFrame_,
        )

    @pytest.fixture
    def add_textbox_fixt(self, spTree, CT_Shape_, insert_element_before_, sp_):
        id_, name = 42, "name"
        x, y, cx, cy = 3, 4, 5, 6
        return (spTree, id_, name, x, y, cx, cy, CT_Shape_, insert_element_before_, sp_)

    @pytest.fixture(
        params=[
            ("p:grpSp", (0, 0, 0, 0)),
            (
                "p:grpSp/p:sp/p:spPr/a:xfrm/(a:off{x=1,y=2},a:ext{cx=3,cy=4})",
                (1, 2, 3, 4),
            ),
            (
                "p:grpSp/(p:sp/p:spPr/a:xfrm/(a:off{x=1,y=2},a:ext{cx=3,cy=4}),p:sp"
                "/p:spPr/a:xfrm/(a:off{x=10,y=20},a:ext{cx=30,cy=40}))",
                (1, 2, 39, 58),
            ),
            (
                "p:grpSp/(p:sp/p:spPr/a:xfrm/(a:off{x=100,y=50},a:ext{cx=25,cy=25})"
                ",p:sp/p:spPr/a:xfrm/(a:off{x=50,y=100},a:ext{cx=25,cy=25}),p:sp/p:"
                "spPr/a:xfrm/(a:off{x=150,y=75},a:ext{cx=25,cy=25}))",
                (50, 50, 125, 75),
            ),
        ]
    )
    def child_exts_fixture(self, request):
        xSp_cxml, expected_values = request.param
        xSp = element(xSp_cxml)
        return xSp, expected_values

    @pytest.fixture(
        params=[
            ("p:spTree", None, [], "p:spTree"),
            (
                "p:grpSp/p:grpSpPr/a:xfrm",
                (1, 2, 3, 4),
                [call()],
                "p:grpSp/p:grpSpPr/a:xfrm/(a:off{x=1,y=2},a:ext{cx=3,cy=4},a:chOff{"
                "x=1,y=2},a:chExt{cx=3,cy=4})",
            ),
        ]
    )
    def recalc_fixture(self, request, _child_extents_prop_, getparent_, grpSp_):
        xSp_cxml, extents, calls, expected_cxml = request.param
        xSp = element(xSp_cxml)

        _child_extents_prop_.return_value = extents
        expected_xml = xml(expected_cxml)
        getparent_.return_value = parent_sp = grpSp_
        return xSp, expected_xml, parent_sp, calls

    # fixture components -----------------------------------

    @pytest.fixture
    def _child_extents_prop_(self, request):
        return property_mock(request, CT_GroupShape, "_child_extents")

    @pytest.fixture
    def CT_GraphicalObjectFrame_(self, request, graphicFrame_):
        CT_GraphicalObjectFrame_ = class_mock(
            request, "pptx.oxml.shapes.groupshape.CT_GraphicalObjectFrame"
        )
        CT_GraphicalObjectFrame_.new_table_graphicFrame.return_value = graphicFrame_
        return CT_GraphicalObjectFrame_

    @pytest.fixture
    def CT_Picture_(self, request, pic_):
        CT_Picture_ = class_mock(request, "pptx.oxml.shapes.groupshape.CT_Picture")
        CT_Picture_.new_pic.return_value = pic_
        return CT_Picture_

    @pytest.fixture
    def CT_Shape_(self, request, sp_):
        CT_Shape_ = class_mock(request, "pptx.oxml.shapes.groupshape.CT_Shape")
        CT_Shape_.new_autoshape_sp.return_value = sp_
        CT_Shape_.new_placeholder_sp.return_value = sp_
        CT_Shape_.new_textbox_sp.return_value = sp_
        return CT_Shape_

    @pytest.fixture
    def getparent_(self, request):
        return method_mock(request, CT_GroupShape, "getparent")

    @pytest.fixture
    def graphicFrame_(self, request):
        return instance_mock(request, CT_GraphicalObjectFrame)

    @pytest.fixture
    def grpSp_(self, request):
        return instance_mock(request, CT_GroupShape)

    @pytest.fixture
    def insert_element_before_(self, request):
        return method_mock(request, CT_GroupShape, "insert_element_before")

    @pytest.fixture
    def pic_(self, request):
        return instance_mock(request, CT_Picture)

    @pytest.fixture
    def sp_(self, request):
        return instance_mock(request, CT_Shape)

    @pytest.fixture
    def spTree(self):
        return element("p:spTree")



# encoding: utf-8

"""Gherkin step implementations for chart data features."""

from __future__ import absolute_import, division, print_function, unicode_literals

import datetime

from behave import given, then, when

from pptx.chart.data import BubbleChartData, Category, CategoryChartData, XyChartData
from pptx.enum.chart import XL_CHART_TYPE
from pptx.util import Inches


# given ===================================================


@given("a BubbleChartData object with number format {strval}")
def given_a_BubbleChartData_object_with_number_format(context, strval):
    params = {}
    if strval != "None":
        params["number_format"] = int(strval)
    context.chart_data = BubbleChartData(**params)


@given("a Categories object with number format {init_nf}")
def given_a_Categories_object_with_number_format_init_nf(context, init_nf):
    categories = CategoryChartData().categories
    if init_nf != "left as default":
        categories.number_format = init_nf
    context.categories = categories


@given("a Category object")
def given_a_Category_object(context):
    context.category = Category(None, None)


@given("a CategoryChartData object")
def given_a_CategoryChartData_object(context):
    context.chart_data = CategoryChartData()


@given("a CategoryChartData object having date categories")
def given_a_CategoryChartData_object_having_date_categories(context):
    chart_data = CategoryChartData()
    chart_data.categories = [
        datetime.date(2016, 12, 27),
        datetime.date(2016, 12, 28),
        datetime.date(2016, 12, 29),
    ]
    context.chart_data = chart_data


@given("a CategoryChartData object with number format {strval}")
def given_a_CategoryChartData_object_with_number_format(context, strval):
    params = {}
    if strval != "None":
        params["number_format"] = int(strval)
    context.chart_data = CategoryChartData(**params)


@given("a XyChartData object with number format {strval}")
def given_a_XyChartData_object_with_number_format(context, strval):
    params = {}
    if strval != "None":
        params["number_format"] = int(strval)
    context.chart_data = XyChartData(**params)


@given("the categories are of type {type_}")
def given_the_categories_are_of_type(context, type_):
    label = {
        "date": datetime.date(2016, 12, 22),
        "float": 42.24,
        "int": 42,
        "str": "foobar",
    }[type_]
    context.categories.add_category(label)


# when ====================================================


@when("I add a bubble data point with number format {strval}")
def when_I_add_a_bubble_data_point_with_number_format(context, strval):
    series_data = context.series_data
    params = {"x": 1, "y": 2, "size": 10}
    if strval != "None":
        params["number_format"] = int(strval)
    context.data_point = series_data.add_data_point(**params)


@when("I add a data point with number format {strval}")
def when_I_add_a_data_point_with_number_format(context, strval):
    series_data = context.series_data
    params = {"value": 42}
    if strval != "None":
        params["number_format"] = int(strval)
    context.data_point = series_data.add_data_point(**params)


@when("I add an XY data point with number format {strval}")
def when_I_add_an_XY_data_point_with_number_format(context, strval):
    series_data = context.series_data
    params = {"x": 1, "y": 2}
    if strval != "None":
        params["number_format"] = int(strval)
    context.data_point = series_data.add_data_point(**params)


@when("I add an {xy_type} chart having 2 series of 3 points each")
def when_I_add_an_xy_chart_having_2_series_of_3_points(context, xy_type):
    chart_type = getattr(XL_CHART_TYPE, xy_type)
    data = (
        ("Series 1", ((-0.1, 0.5), (16.2, 0.0), (8.0, 0.2))),
        ("Series 2", ((12.4, 0.8), (-7.5, -0.5), (-5.1, -0.2))),
    )

    chart_data = XyChartData()

    for series_data in data:
        series_label, points = series_data
        series = chart_data.add_series(series_label)
        for point in points:
            x, y = point
            series.add_data_point(x, y)

    context.chart = context.slide.shapes.add_chart(
        chart_type, Inches(1), Inches(1), Inches(8), Inches(5), chart_data
    ).chart


@when("I assign ['a', 'b', 'c'] to chart_data.categories")
def when_I_assign_a_b_c_to_chart_data_categories(context):
    chart_data = context.chart_data
    chart_data.categories = ["a", "b", "c"]


# then ====================================================


@then("[c.label for c in chart_data.categories] is ['a', 'b', 'c']")
def then_c_label_for_c_in_chart_data_categories_is_a_b_c(context):
    chart_data = context.chart_data
    assert [c.label for c in chart_data.categories] == ["a", "b", "c"]


@then("categories.number_format is {value}")
def then_categories_number_format_is_value(context, value):
    expected_value = value
    number_format = context.categories.number_format
    assert number_format == expected_value, "got %s" % number_format


@then("category.add_sub_category(name) is a Category object")
def then_category_add_sub_category_is_a_Category_object(context):
    category = context.category
    context.sub_category = sub_category = category.add_sub_category("foobar")
    assert type(sub_category).__name__ == "Category"


@then("category.sub_categories[-1] is the new category")
def then_category_sub_categories_minus_1_is_the_new_category(context):
    category, sub_category = context.category, context.sub_category
    assert category.sub_categories[-1] is sub_category


@then("chart_data.add_category(name) is a Category object")
def then_chart_data_add_category_name_is_a_Category_object(context):
    chart_data = context.chart_data
    context.category = category = chart_data.add_category("foobar")
    assert type(category).__name__ == "Category"


@then("chart_data.add_series(name, values) is a CategorySeriesData object")
def then_chart_data_add_series_is_a_CategorySeriesData_object(context):
    chart_data = context.chart_data
    context.series = series = chart_data.add_series("Series X", (1, 2, 3))
    assert type(series).__name__ == "CategorySeriesData"


@then("chart_data.categories is a Categories object")
def then_chart_data_categories_is_a_Categories_object(context):
    chart_data = context.chart_data
    assert type(chart_data.categories).__name__ == "Categories"


@then("chart_data.categories[-1] is the category")
def then_chart_data_categories_minus_1_is_the_category(context):
    chart_data, category = context.chart_data, context.category
    assert chart_data.categories[-1] is category


@then("chart_data.number_format is {value_str}")
def then_chart_data_number_format_is(context, value_str):
    chart_data = context.chart_data
    number_format = value_str if value_str == "General" else int(value_str)
    assert chart_data.number_format == number_format


@then("chart_data[-1] is the new series")
def then_chart_data_minus_1_is_the_new_series(context):
    chart_data, series = context.chart_data, context.series
    assert chart_data[-1] is series


@then("series_data.number_format is {value_str}")
def then_series_data_number_format_is(context, value_str):
    series_data = context.series_data
    number_format = value_str if value_str == "General" else int(value_str)
    assert series_data.number_format == number_format



# encoding: utf-8

"""Gherkin step implementations for chart legend features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, then, when

from pptx import Presentation
from pptx.enum.chart import XL_LEGEND_POSITION
from pptx.text.text import Font

from helpers import test_pptx


# given ===================================================


@given("a legend")
def given_a_legend(context):
    prs = Presentation(test_pptx("cht-legend-props"))
    context.legend = prs.slides[0].shapes[0].chart.legend


@given("a legend having horizontal offset of {value}")
def given_a_legend_having_horizontal_offset_of_value(context, value):
    slide_idx = {"none": 0, "-0.5": 1, "0.42": 2}[value]
    prs = Presentation(test_pptx("cht-legend-props"))
    context.legend = prs.slides[slide_idx].shapes[0].chart.legend


@given("a legend positioned {location} the chart")
def given_a_legend_positioned_location_the_chart(context, location):
    slide_idx = {"at an unspecified location of": 0, "below": 1, "to the right of": 2}[
        location
    ]
    prs = Presentation(test_pptx("cht-legend-props"))
    context.legend = prs.slides[slide_idx].shapes[0].chart.legend


@given("a legend with overlay setting of {setting}")
def given_a_legend_with_overlay_setting_of_setting(context, setting):
    slide_idx = {"no explicit setting": 0, "True": 1, "False": 2}[setting]
    prs = Presentation(test_pptx("cht-legend-props"))
    context.legend = prs.slides[slide_idx].shapes[0].chart.legend


# when ====================================================


@when("I assign {value} to legend.horz_offset")
def when_I_assign_value_to_legend_horz_offset(context, value):
    new_value = float(value)
    context.legend.horz_offset = new_value


@when("I assign {value} to legend.include_in_layout")
def when_I_assign_value_to_legend_include_in_layout(context, value):
    new_value = {"True": True, "False": False}[value]
    context.legend.include_in_layout = new_value


@when("I assign {value} to legend.position")
def when_I_assign_value_to_legend_position(context, value):
    enum_value = getattr(XL_LEGEND_POSITION, value)
    context.legend.position = enum_value


# then ====================================================


@then("legend.font is a Font object")
def then_legend_font_is_a_Font_object(context):
    legend = context.legend
    assert isinstance(legend.font, Font)


@then("legend.horz_offset is {value}")
def then_legend_horz_offset_is_value(context, value):
    expected_value = float(value)
    legend = context.legend
    assert legend.horz_offset == expected_value


@then("legend.include_in_layout is {value}")
def then_legend_include_in_layout_is_value(context, value):
    expected_value = {"True": True, "False": False}[value]
    legend = context.legend
    assert legend.include_in_layout is expected_value


@then("legend.position is {value}")
def then_legend_position_is_value(context, value):
    expected_position = getattr(XL_LEGEND_POSITION, value)
    legend = context.legend
    assert legend.position is expected_position, "got %s" % legend.position



# encoding: utf-8

"""
Legend of a chart.
"""

from __future__ import absolute_import, print_function, unicode_literals

from ..enum.chart import XL_LEGEND_POSITION
from ..text.text import Font
from ..util import lazyproperty


class Legend(object):
    """
    Represents the legend in a chart. A chart can have at most one legend.
    """

    def __init__(self, legend_elm):
        super(Legend, self).__init__()
        self._element = legend_elm

    @lazyproperty
    def font(self):
        """
        The |Font| object that provides access to the text properties for
        this legend, such as bold, italic, etc.
        """
        defRPr = self._element.defRPr
        font = Font(defRPr)
        return font

    @property
    def horz_offset(self):
        """
        Adjustment of the x position of the legend from its default.
        Expressed as a float between -1.0 and 1.0 representing a fraction of
        the chart width. Negative values move the legend left, positive
        values move it to the right. |None| if no setting is specified.
        """
        return self._element.horz_offset

    @horz_offset.setter
    def horz_offset(self, value):
        self._element.horz_offset = value

    @property
    def include_in_layout(self):
        """|True| if legend should be located inside plot area.

        Read/write boolean specifying whether legend should be placed inside
        the plot area. In many cases this will cause it to be superimposed on
        the chart itself. Assigning |None| to this property causes any
        `c:overlay` element to be removed, which is interpreted the same as
        |True|. This use case should rarely be required and assigning
        a boolean value is recommended.
        """
        overlay = self._element.overlay
        if overlay is None:
            return True
        return overlay.val

    @include_in_layout.setter
    def include_in_layout(self, value):
        if value is None:
            self._element._remove_overlay()
            return
        self._element.get_or_add_overlay().val = bool(value)

    @property
    def position(self):
        """
        Read/write :ref:`XlLegendPosition` enumeration value specifying the
        general region of the chart in which to place the legend.
        """
        legendPos = self._element.legendPos
        if legendPos is None:
            return XL_LEGEND_POSITION.RIGHT
        return legendPos.val

    @position.setter
    def position(self, position):
        self._element.get_or_add_legendPos().val = position



# encoding: utf-8

"""Table-related objects such as Table and Cell."""

from __future__ import absolute_import, division, print_function, unicode_literals

from pptx.compat import is_integer
from pptx.dml.fill import FillFormat
from pptx.oxml.table import TcRange
from pptx.shapes import Subshape
from pptx.text.text import TextFrame
from pptx.util import lazyproperty


class Table(object):
    """A DrawingML table object.

    Not intended to be constructed directly, use
    :meth:`.Slide.shapes.add_table` to add a table to a slide.
    """

    def __init__(self, tbl, graphic_frame):
        super(Table, self).__init__()
        self._tbl = tbl
        self._graphic_frame = graphic_frame

    def cell(self, row_idx, col_idx):
        """Return cell at *row_idx*, *col_idx*.

        Return value is an instance of |_Cell|. *row_idx* and *col_idx* are
        zero-based, e.g. cell(0, 0) is the top, left cell in the table.
        """
        return _Cell(self._tbl.tc(row_idx, col_idx), self)

    @lazyproperty
    def columns(self):
        """
        Read-only reference to collection of |_Column| objects representing
        the table's columns. |_Column| objects are accessed using list
        notation, e.g. ``col = tbl.columns[0]``.
        """
        return _ColumnCollection(self._tbl, self)

    @property
    def first_col(self):
        """
        Read/write boolean property which, when true, indicates the first
        column should be formatted differently, as for a side-heading column
        at the far left of the table.
        """
        return self._tbl.firstCol

    @first_col.setter
    def first_col(self, value):
        self._tbl.firstCol = value

    @property
    def first_row(self):
        """
        Read/write boolean property which, when true, indicates the first
        row should be formatted differently, e.g. for column headings.
        """
        return self._tbl.firstRow

    @first_row.setter
    def first_row(self, value):
        self._tbl.firstRow = value

    @property
    def horz_banding(self):
        """
        Read/write boolean property which, when true, indicates the rows of
        the table should appear with alternating shading.
        """
        return self._tbl.bandRow

    @horz_banding.setter
    def horz_banding(self, value):
        self._tbl.bandRow = value

    def iter_cells(self):
        """Generate _Cell object for each cell in this table.

        Each grid cell is generated in left-to-right, top-to-bottom order.
        """
        return (_Cell(tc, self) for tc in self._tbl.iter_tcs())

    @property
    def last_col(self):
        """
        Read/write boolean property which, when true, indicates the last
        column should be formatted differently, as for a row totals column at
        the far right of the table.
        """
        return self._tbl.lastCol

    @last_col.setter
    def last_col(self, value):
        self._tbl.lastCol = value

    @property
    def last_row(self):
        """
        Read/write boolean property which, when true, indicates the last
        row should be formatted differently, as for a totals row at the
        bottom of the table.
        """
        return self._tbl.lastRow

    @last_row.setter
    def last_row(self, value):
        self._tbl.lastRow = value

    def notify_height_changed(self):
        """
        Called by a row when its height changes, triggering the graphic frame
        to recalculate its total height (as the sum of the row heights).
        """
        new_table_height = sum([row.height for row in self.rows])
        self._graphic_frame.height = new_table_height

    def notify_width_changed(self):
        """
        Called by a column when its width changes, triggering the graphic
        frame to recalculate its total width (as the sum of the column
        widths).
        """
        new_table_width = sum([col.width for col in self.columns])
        self._graphic_frame.width = new_table_width

    @property
    def part(self):
        """
        The package part containing this table.
        """
        return self._graphic_frame.part

    @lazyproperty
    def rows(self):
        """
        Read-only reference to collection of |_Row| objects representing the
        table's rows. |_Row| objects are accessed using list notation, e.g.
        ``col = tbl.rows[0]``.
        """
        return _RowCollection(self._tbl, self)

    @property
    def vert_banding(self):
        """
        Read/write boolean property which, when true, indicates the columns
        of the table should appear with alternating shading.
        """
        return self._tbl.bandCol

    @vert_banding.setter
    def vert_banding(self, value):
        self._tbl.bandCol = value


class _Cell(Subshape):
    """Table cell"""

    def __init__(self, tc, parent):
        super(_Cell, self).__init__(parent)
        self._tc = tc

    def __eq__(self, other):
        """|True| if this object proxies the same element as *other*.

        Equality for proxy objects is defined as referring to the same XML
        element, whether or not they are the same proxy object instance.
        """
        if not isinstance(other, type(self)):
            return False
        return self._tc is other._tc

    def __ne__(self, other):
        if not isinstance(other, type(self)):
            return True
        return self._tc is not other._tc

    @lazyproperty
    def fill(self):
        """
        |FillFormat| instance for this cell, providing access to fill
        properties such as foreground color.
        """
        tcPr = self._tc.get_or_add_tcPr()
        return FillFormat.from_fill_parent(tcPr)

    @property
    def is_merge_origin(self):
        """True if this cell is the top-left grid cell in a merged cell."""
        return self._tc.is_merge_origin

    @property
    def is_spanned(self):
        """True if this cell is spanned by a merge-origin cell.

        A merge-origin cell "spans" the other grid cells in its merge range,
        consuming their area and "shadowing" the spanned grid cells.

        Note this value is |False| for a merge-origin cell. A merge-origin
        cell spans other grid cells, but is not itself a spanned cell.
        """
        return self._tc.is_spanned

    @property
    def margin_left(self):
        """
        Read/write integer value of left margin of cell as a |Length| value
        object. If assigned |None|, the default value is used, 0.1 inches for
        left and right margins and 0.05 inches for top and bottom.
        """
        return self._tc.marL

    @margin_left.setter
    def margin_left(self, margin_left):
        self._validate_margin_value(margin_left)
        self._tc.marL = margin_left

    @property
    def margin_right(self):
        """
        Right margin of cell.
        """
        return self._tc.marR

    @margin_right.setter
    def margin_right(self, margin_right):
        self._validate_margin_value(margin_right)
        self._tc.marR = margin_right

    @property
    def margin_top(self):
        """
        Top margin of cell.
        """
        return self._tc.marT

    @margin_top.setter
    def margin_top(self, margin_top):
        self._validate_margin_value(margin_top)
        self._tc.marT = margin_top

    @property
    def margin_bottom(self):
        """
        Bottom margin of cell.
        """
        return self._tc.marB

    @margin_bottom.setter
    def margin_bottom(self, margin_bottom):
        self._validate_margin_value(margin_bottom)
        self._tc.marB = margin_bottom

    def merge(self, other_cell):
        """Create merged cell from this cell to *other_cell*.

        This cell and *other_cell* specify opposite corners of the merged
        cell range. Either diagonal of the cell region may be specified in
        either order, e.g. self=bottom-right, other_cell=top-left, etc.

        Raises |ValueError| if the specified range already contains merged
        cells anywhere within its extents or if *other_cell* is not in the
        same table as *self*.
        """
        tc_range = TcRange(self._tc, other_cell._tc)

        if not tc_range.in_same_table:
            raise ValueError("other_cell from different table")
        if tc_range.contains_merged_cell:
            raise ValueError("range contains one or more merged cells")

        tc_range.move_content_to_origin()

        row_count, col_count = tc_range.dimensions

        for tc in tc_range.iter_top_row_tcs():
            tc.rowSpan = row_count
        for tc in tc_range.iter_left_col_tcs():
            tc.gridSpan = col_count
        for tc in tc_range.iter_except_left_col_tcs():
            tc.hMerge = True
        for tc in tc_range.iter_except_top_row_tcs():
            tc.vMerge = True

    @property
    def span_height(self):
        """int count of rows spanned by this cell.

        The value of this property may be misleading (often 1) on cells where
        `.is_merge_origin` is not |True|, since only a merge-origin cell
        contains complete span information. This property is only intended
        for use on cells known to be a merge origin by testing
        `.is_merge_origin`.
        """
        return self._tc.rowSpan

    @property
    def span_width(self):
        """int count of columns spanned by this cell.

        The value of this property may be misleading (often 1) on cells where
        `.is_merge_origin` is not |True|, since only a merge-origin cell
        contains complete span information. This property is only intended
        for use on cells known to be a merge origin by testing
        `.is_merge_origin`.
        """
        return self._tc.gridSpan

    def split(self):
        """Remove merge from this (merge-origin) cell.

        The merged cell represented by this object will be "unmerged",
        yielding a separate unmerged cell for each grid cell previously
        spanned by this merge.

        Raises |ValueError| when this cell is not a merge-origin cell. Test
        with `.is_merge_origin` before calling.
        """
        if not self.is_merge_origin:
            raise ValueError(
                "not a merge-origin cell; only a merge-origin cell can be sp" "lit"
            )

        tc_range = TcRange.from_merge_origin(self._tc)

        for tc in tc_range.iter_tcs():
            tc.rowSpan = tc.gridSpan = 1
            tc.hMerge = tc.vMerge = False

    @property
    def text(self):
        """Unicode (str in Python 3) representation of cell contents.

        The returned string will contain a newline character (``"\\n"``) separating each
        paragraph and a vertical-tab (``"\\v"``) character for each line break (soft
        carriage return) in the cell's text.

        Assignment to *text* replaces all text currently contained in the cell. A
        newline character (``"\\n"``) in the assigned text causes a new paragraph to be
        started. A vertical-tab (``"\\v"``) character in the assigned text causes
        a line-break (soft carriage-return) to be inserted. (The vertical-tab character
        appears in clipboard text copied from PowerPoint as its encoding of
        line-breaks.)

        Either bytes (Python 2 str) or unicode (Python 3 str) can be assigned. Bytes can
        be 7-bit ASCII or UTF-8 encoded 8-bit bytes. Bytes values are converted to
        unicode assuming UTF-8 encoding (which correctly decodes ASCII).
        """
        return self.text_frame.text

    @text.setter
    def text(self, text):
        self.text_frame.text = text

    @property
    def text_frame(self):
        """
        |TextFrame| instance containing the text that appears in the cell.
        """
        txBody = self._tc.get_or_add_txBody()
        return TextFrame(txBody, self)

    @property
    def vertical_anchor(self):
        """Vertical alignment of this cell.

        This value is a member of the :ref:`MsoVerticalAnchor` enumeration or
        |None|. A value of |None| indicates the cell has no explicitly
        applied vertical anchor setting and its effective value is inherited
        from its style-hierarchy ancestors.

        Assigning |None| to this property causes any explicitly applied
        vertical anchor setting to be cleared and inheritance of its
        effective value to be restored.
        """
        return self._tc.anchor

    @vertical_anchor.setter
    def vertical_anchor(self, mso_anchor_idx):
        self._tc.anchor = mso_anchor_idx

    @staticmethod
    def _validate_margin_value(margin_value):
        """
        Raise ValueError if *margin_value* is not a positive integer value or
        |None|.
        """
        if not is_integer(margin_value) and margin_value is not None:
            tmpl = "margin value must be integer or None, got '%s'"
            raise TypeError(tmpl % margin_value)


class _Column(Subshape):
    """Table column"""

    def __init__(self, gridCol, parent):
        super(_Column, self).__init__(parent)
        self._gridCol = gridCol

    @property
    def width(self):
        """
        Width of column in EMU.
        """
        return self._gridCol.w

    @width.setter
    def width(self, width):
        self._gridCol.w = width
        self._parent.notify_width_changed()


class _Row(Subshape):
    """Table row"""

    def __init__(self, tr, parent):
        super(_Row, self).__init__(parent)
        self._tr = tr

    @property
    def cells(self):
        """
        Read-only reference to collection of cells in row. An individual cell
        is referenced using list notation, e.g. ``cell = row.cells[0]``.
        """
        return _CellCollection(self._tr, self)

    @property
    def height(self):
        """
        Height of row in EMU.
        """
        return self._tr.h

    @height.setter
    def height(self, height):
        self._tr.h = height
        self._parent.notify_height_changed()


class _CellCollection(Subshape):
    """Horizontal sequence of row cells"""

    def __init__(self, tr, parent):
        super(_CellCollection, self).__init__(parent)
        self._tr = tr

    def __getitem__(self, idx):
        """Provides indexed access, (e.g. 'cells[0]')."""
        if idx < 0 or idx >= len(self._tr.tc_lst):
            msg = "cell index [%d] out of range" % idx
            raise IndexError(msg)
        return _Cell(self._tr.tc_lst[idx], self)

    def __iter__(self):
        """Provides iterability."""
        return (_Cell(tc, self) for tc in self._tr.tc_lst)

    def __len__(self):
        """Supports len() function (e.g. 'len(cells) == 1')."""
        return len(self._tr.tc_lst)


class _ColumnCollection(Subshape):
    """Sequence of table columns."""

    def __init__(self, tbl, parent):
        super(_ColumnCollection, self).__init__(parent)
        self._tbl = tbl

    def __getitem__(self, idx):
        """
        Provides indexed access, (e.g. 'columns[0]').
        """
        if idx < 0 or idx >= len(self._tbl.tblGrid.gridCol_lst):
            msg = "column index [%d] out of range" % idx
            raise IndexError(msg)
        return _Column(self._tbl.tblGrid.gridCol_lst[idx], self)

    def __len__(self):
        """
        Supports len() function (e.g. 'len(columns) == 1').
        """
        return len(self._tbl.tblGrid.gridCol_lst)

    def notify_width_changed(self):
        """
        Called by a column when its width changes. Pass along to parent.
        """
        self._parent.notify_width_changed()


class _RowCollection(Subshape):
    """Sequence of table rows"""

    def __init__(self, tbl, parent):
        super(_RowCollection, self).__init__(parent)
        self._tbl = tbl

    def __getitem__(self, idx):
        """
        Provides indexed access, (e.g. 'rows[0]').
        """
        if idx < 0 or idx >= len(self):
            msg = "row index [%d] out of range" % idx
            raise IndexError(msg)
        return _Row(self._tbl.tr_lst[idx], self)

    def __len__(self):
        """
        Supports len() function (e.g. 'len(rows) == 1').
        """
        return len(self._tbl.tr_lst)

    def notify_height_changed(self):
        """
        Called by a row when its height changes. Pass along to parent.
        """
        self._parent.notify_height_changed()
